
IF(DEFINED NLIB_ONCE_COMMON_CMAKE)
  RETURN()
ENDIF()
SET(NLIB_ONCE_COMMON_CMAKE True)

IF(${CMAKE_BUILD_TYPE} MATCHES "^$")
	SET(CMAKE_BUILD_TYPE "Release")
ENDIF()

CMAKE_POLICY(SET CMP0022 NEW)
CMAKE_POLICY(SET CMP0038 NEW)
CMAKE_POLICY(SET CMP0042 NEW)
CMAKE_POLICY(SET CMP0054 NEW)
SET(CMAKE_WARN_DEPRECATED TRUE)

IF(NOT NLIB_PKGROOT_DIR)
	SET(NLIB_PKGROOT_DIR ${NLIB_RELEASE_ROOT})
ENDIF()

SET(CMAKE_EXPORT_COMPILE_COMMANDS ON)

IF(DEFINED ENV{NLIB_CFUNC_PREFIX})
	SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_DEFINITIONS "NLIB_RENAME_CAPI;NLIB_CFUNC_PREFIX=$ENV{NLIB_CFUNC_PREFIX}")
ENDIF()

IF(DEFINED ENV{NLIB_NAMESPACE_BEGIN} AND
   DEFINED ENV{NLIB_NAMESPACE_END} AND
   DEFINED ENV{NLIB_NS})
   # ex. export NLIB_NAMESPACE_BEGIN="namespace ci {"
   # ex. export NLIB_NAMESPACE_END="}"
   # ex. export NLIB_NS="::ci"
   IF(CYGWIN AND (CAFE OR CTR OR NX))
       SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_DEFINITIONS "NLIB_NAMESPACE_BEGIN=\"$ENV{NLIB_NAMESPACE_BEGIN}\"")
   	   SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_DEFINITIONS "NLIB_NAMESPACE_END=\"$ENV{NLIB_NAMESPACE_END}\"")
       SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_DEFINITIONS "NLIB_NS=\"$ENV{NLIB_NS}\"")
   ELSE()
       SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_DEFINITIONS "NLIB_NAMESPACE_BEGIN=$ENV{NLIB_NAMESPACE_BEGIN}")
       SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_DEFINITIONS "NLIB_NAMESPACE_END=$ENV{NLIB_NAMESPACE_END}")
       SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_DEFINITIONS "NLIB_NS=$ENV{NLIB_NS}")
   ENDIF()
ENDIF()

# environment variable defined by cibuild like jenkins
IF(DEFINED ENV{NLIB_CIBUILD})
	SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_DEFINITIONS "NLIB_CIBUILD")
ENDIF()

IF(DEFINED ENV{WERROR} AND NOT MSVC_VERSION)
	SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_OPTIONS "-Werror")
ENDIF()

IF(CYGWIN OR ${CMAKE_HOST_SYSTEM_NAME} MATCHES Cygwin)
	FIND_PROGRAM(CYGPATH cygpath)
	IF(NOT CYGPATH)
		MESSAGE(FATAL_ERROR "!!!!! cannot find cygpath !!!!!")
	ENDIF()
	FUNCTION(nlib_cygwin_getcygpath _from _to)
		EXECUTE_PROCESS(COMMAND
			${CYGPATH} -u "${_from}"
			RESULT_VARIABLE cygpath_result
			ERROR_VARIABLE cygpath_err
			OUTPUT_VARIABLE _toraw
		)
		IF(cygpath_result OR cygpath_err)
			MESSAGE(FATAL_ERROR "${CYGPATH} failed:: ${cygpath_err}")
		ENDIF()
		STRING(REGEX REPLACE "[\r\n]" "" _toans "${_toraw}")
		GET_FILENAME_COMPONENT(_toans ${_toans} ABSOLUTE)
		SET(${_to} ${_toans} PARENT_SCOPE)
	ENDFUNCTION()

	FUNCTION(nlib_cygwin_getwinpath _from _to)
		GET_FILENAME_COMPONENT(_from ${_from} ABSOLUTE)
		EXECUTE_PROCESS(COMMAND
			${CYGPATH} -m "${_from}"
			RESULT_VARIABLE cygpath_result
			ERROR_VARIABLE cygpath_err
			OUTPUT_VARIABLE _toraw
		)
		IF(cygpath_result OR cygpath_err)
			MESSAGE(FATAL_ERROR "${CYGPATH} failed:: ${cygpath_err}")
		ENDIF()
		STRING(REGEX REPLACE "[\r\n]" "" _toans "${_toraw}")
		SET(${_to} ${_toans} PARENT_SCOPE)
	ENDFUNCTION()
ELSE()
	FUNCTION(nlib_cygwin_getcygpath _from _to)
		SET(${_to} ${_from} PARENT_SCOPE)
	ENDFUNCTION()
	FUNCTION(nlib_cygwin_getwinpath _from _to)
		SET(${_to} ${_from} PARENT_SCOPE)
	ENDFUNCTION()
ENDIF()

FUNCTION(join VALUES GLUE OUTPUT)
  STRING(REPLACE ";" "${GLUE}" _TMP_STR "${VALUES}")
  SET(${OUTPUT} "${_TMP_STR}" PARENT_SCOPE)
ENDFUNCTION()

IF(NX)
	INCLUDE("${CMAKE_CURRENT_LIST_DIR}/common.nx.cmake")
ELSEIF(CAFE)
	INCLUDE("${CMAKE_CURRENT_LIST_DIR}/common.cafe.cmake")
ELSEIF(CTR)
	INCLUDE("${CMAKE_CURRENT_LIST_DIR}/common.ctr.cmake")
ELSEIF(WIN32 OR CYGWIN OR UNIX)
	SET(NLIB_PCDEV 1)
	INCLUDE("${CMAKE_CURRENT_LIST_DIR}/common.pc.cmake")
ELSE()
	MESSAGE(FATAL_ERROR "!!!!! Sorry, Platform not supported !!!!!")
ENDIF()

IF(NLIB_APPLE_DIR_WORKAROUND)
	SET(NLIB_TOOL_DIR ${NLIB_RELEASE_ROOT}/bin/cmake/clang/Release)
ELSEIF(NLIB_PCDEV)
	SET(NLIB_TOOL_DIR ${NLIB_RELEASE_ROOT}/bin/cmake/${CAT_DIR}/${CMAKE_BUILD_TYPE})
ELSEIF(WIN32 OR NX OR CTR OR CAFE)
	IF(EXISTS ${NLIB_RELEASE_ROOT}/bin/cmake/vc150amd64/Release)
		SET(NLIB_TOOL_DIR ${NLIB_RELEASE_ROOT}/bin/cmake/vc150amd64/Release)
	ELSEIF(EXISTS ${NLIB_RELEASE_ROOT}/bin/cmake/vc140amd64/Release)
		SET(NLIB_TOOL_DIR ${NLIB_RELEASE_ROOT}/bin/cmake/vc140amd64/Release)
	ELSEIF(EXISTS ${NLIB_RELEASE_ROOT}/bin/cmake/vc140/Release)
		SET(NLIB_TOOL_DIR ${NLIB_RELEASE_ROOT}/bin/cmake/vc140/Release)
	ELSEIF(EXISTS ${NLIB_RELEASE_ROOT}/bin/cmake/vc120amd64/Release)
		SET(NLIB_TOOL_DIR ${NLIB_RELEASE_ROOT}/bin/cmake/vc120amd64/Release)
	ELSEIF(EXISTS ${NLIB_RELEASE_ROOT}/bin/cmake/vc120/Release)
		SET(NLIB_TOOL_DIR ${NLIB_RELEASE_ROOT}/bin/cmake/vc120/Release)
	ELSE()
		MESSAGE(FATAL_ERROR "cannot set NLIB_TOOL_DIR")
	ENDIF()
ELSEIF(CYGWIN)
	SET(NLIB_TOOL_DIR ${NLIB_RELEASE_ROOT}/bin/cmake/cygwin/Release)
ELSE()
	IF(EXISTS ${NLIB_RELEASE_ROOT}/bin/cmake/gcc/Release)
		SET(NLIB_TOOL_DIR ${NLIB_RELEASE_ROOT}/bin/cmake/gcc/Release)
	ELSE()
		SET(NLIB_TOOL_DIR ${NLIB_RELEASE_ROOT}/bin/cmake/clang/Release)
	ENDIF()
ENDIF()

SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_DEFINITIONS "$<$<CONFIG:Release>:NDEBUG>")

MESSAGE(STATUS "NLIB_TOOL_DIR=${NLIB_TOOL_DIR}")
MESSAGE(STATUS "NLIB_DIR=${NLIB_DIR}")
MESSAGE(STATUS "CMAKE_BUILD_TYPE : ${CMAKE_BUILD_TYPE}")
MESSAGE(STATUS "CMAKE_CXX_COMPILER_ID : ${CMAKE_CXX_COMPILER_ID}")
MESSAGE(STATUS "CMAKE_CXX_FLAGS : ${CMAKE_CXX_FLAGS}")
MESSAGE(STATUS "CMAKE_CXX_FLAGS_DEBUG : ${CMAKE_CXX_FLAGS_DEBUG}")
MESSAGE(STATUS "CMAKE_CXX_FLAGS_RELWITHDEBINFO : ${CMAKE_CXX_FLAGS_RELWITHDEBINFO}")
MESSAGE(STATUS "CMAKE_CXX_FLAGS_RELEASE : ${CMAKE_CXX_FLAGS_RELEASE}")
MESSAGE(STATUS "CMAKE_C_FLAGS : ${CMAKE_C_FLAGS}")
MESSAGE(STATUS "CMAKE_C_FLAGS_DEBUG : ${CMAKE_C_FLAGS_DEBUG}")
MESSAGE(STATUS "CMAKE_C_FLAGS_RELWITHDEBINFO : ${CMAKE_C_FLAGS_RELWITHDEBINFO}")
MESSAGE(STATUS "CMAKE_C_FLAGS_RELEASE : ${CMAKE_C_FLAGS_RELEASE}")
MESSAGE(STATUS "CMAKE_STANDARD_LIBRARIES : ${CMAKE_STANDARD_LIBRARIES}")
MESSAGE(STATUS "CMAKE_EXE_LINKER_FLAGS : ${CMAKE_EXE_LINKER_FLAGS}")
MESSAGE(STATUS "CMAKE_CXX_LINK_FLAGS: ${CMAKE_CXX_LINK_FLAGS}")
MESSAGE(STATUS "CMAKE_CXX_LINK_EXECUTABLE : ${CMAKE_CXX_LINK_EXECUTABLE}")
MESSAGE(STATUS "CMAKE_DEBUG_POSTFIX : ${CMAKE_DEBUG_POSTFIX}")
MESSAGE(STATUS "CMAKE_RELEASE_POSTFIX : ${CMAKE_RELEASE_POSTFIX}")

# you can check command lines if you uncomment this
# set( CMAKE_VERBOSE_MAKEFILE on )
