﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_CONFIG_WIN32_H_
#define INCLUDE_NN_NLIB_CONFIG_WIN32_H_
#ifndef INCLUDE_NN_NLIB_CONFIG_H_
# error do not include directly
#endif

#if defined(_MSC_VER)

#include <Windows.h>
#include <crtdbg.h>
#if _WIN32_WINNT < 0x601
# pragma message("Windows SDK is old. Please install Windows SDK 7.1 or higher.")
#endif

#define NLIB_PATHMAPPER_FORSAMPLE \
    nlib_ns::NativePathMapper g_pathmapper; \
    nlib_ns::NativePathMapper& g_PathMapper = g_pathmapper; \
    inline void InitPathMapperForSample() { \
        char dir[1024]; \
        ::GetModuleFileNameA(NULL, &dir[0], 1024); \
        *const_cast<char*>(nlib_strrchr(dir, '\\')) = '\0'; \
        *const_cast<char*>(nlib_strrchr(dir, '\\')) = '\0'; \
        g_pathmapper.AddMap(dir, "nlibpath:///readonly/"); \
        g_pathmapper.AddMap(dir, "nlibpath:///readwrite/"); \
    }

#define NLIB_MAINFUNC int main(int argc, char** argv) { \
    bool result = SampleMain(argc, argv); nlib_printf(result ? "OK\n" : "NG\n"); \
    return result ? 0 : 1; }
#define NLIB_EXCEPTION_ENABLED

#if 0
// Klocwork likes it better than crtdbg.h
#include <cassert> // NOLINT
#define NLIB_ASSERT assert
#else
#define NLIB_ASSERT _ASSERT
#endif

#ifdef _DEBUG
#define NLIB_ASSERT_NOERR(e)                                           \
    (void) (((e) == 0) ||                                              \
            (1 != _CrtDbgReport(_CRT_ASSERT, __FILE__, __LINE__, NULL, \
                                "%s", nlib_error_string(e))) ||        \
            (_CrtDbgBreak(), 0))
#else
#define NLIB_ASSERT_NOERR(e) ((void)0)
#endif

#define NLIB_ALIGNAS(x) __declspec(align(x))
#define NLIB_ALIGNOF(tp) __alignof(tp)
#endif

#ifdef _DEBUG
NLIB_NAMESPACE_BEGIN

class NlibMemChecker {
 public:
    __declspec(noinline) NlibMemChecker() {
        _CrtMemCheckpoint(&s1_);
    }
    __declspec(noinline) void Start() {
        _CrtMemCheckpoint(&s1_);
    }
    __declspec(noinline) bool Check() {
        _CrtMemState s2, s3;
        _CrtMemCheckpoint(&s2);
        return !_CrtMemDifference( &s3, &s1_, &s2);
    }

 private:
    _CrtMemState s1_;
};

#define NLIB_MEMCHECKER ::nlib_ns::NlibMemChecker nlib_memchecker_
#define NLIB_MEMCHECKER_START nlib_memchecker_.Start()
#define NLIB_MEMCHECKER_CHECK nlib_memchecker_.Check()

NLIB_NAMESPACE_END
#endif

#define NLIB_NOEMPTYFILE_2(L) namespace { char NoEmptyFileDummy ## L; } // NOLINT
#define NLIB_NOEMPTYFILE_1(L) NLIB_NOEMPTYFILE_2(L)
#define NLIB_NOEMPTYFILE() NLIB_NOEMPTYFILE_1(__LINE__)

#define NLIB_HAS_VIRTUALMEMORY

#define NLIB_THREAD_EXIT_CPP_THROW_NLIB_EXCEPTION
#ifdef NLIB_EXCEPTION_ENABLED
#undef nlib_thread_cleanup_push
#undef nlib_thread_cleanup_pop
#define nlib_thread_cleanup_push(fn, arg) switch (0) case 0: default: { \
    __pragma(warning(push)) \
    __pragma(warning(disable:4456)) \
    struct nlib_thread_cleanup_handler_ _thread_cleanup_handler = { fn, arg, NULL }; \
    __pragma(warning(pop)) \
    nlib_thread_cleanup_push_(&_thread_cleanup_handler); try {
#define nlib_thread_cleanup_pop(exec) } catch(...) { nlib_thread_cleanup_pop_(1); throw; } \
    nlib_thread_cleanup_pop_(exec); }
#endif

#endif  // INCLUDE_NN_NLIB_CONFIG_WIN32_H_
