﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_FILEINPUTSTREAM_H_
#define INCLUDE_NN_NLIB_FILEINPUTSTREAM_H_

#include "nn/nlib/InputStream.h"

NLIB_NAMESPACE_BEGIN

struct FileInputStreamSettings {
    void* buf;
    size_t buffer_size;

 public:
    NLIB_CEXPR FileInputStreamSettings() NLIB_NOEXCEPT : buf(NULL), buffer_size(4096) {}
    NLIB_CEXPR FileInputStreamSettings(void* buf_, size_t buffer_size_) NLIB_NOEXCEPT
        : buf(buf_),
          buffer_size(buffer_size_) {}
};

class NLIB_VIS_PUBLIC FileInputStream NLIB_FINAL : public InputStream {
 public:
    FileInputStream() NLIB_NOEXCEPT : fd_(NLIB_FD_INVALID),
                                      buf_(NULL),
                                      buf_size_(0),
                                      is_internal_buf_(false) {}
    virtual ~FileInputStream() NLIB_NOEXCEPT NLIB_OVERRIDE;
    errno_t Init() NLIB_NOEXCEPT {
        NLIB_CEXPR FileInputStreamSettings settings;
        return this->Init(settings);
    }
    NLIB_CHECK_RESULT errno_t Init(const FileInputStreamSettings& settings) NLIB_NOEXCEPT;
    NLIB_CHECK_RESULT errno_t Open(const char* filename) NLIB_NOEXCEPT NLIB_NONNULL;
    NLIB_CHECK_RESULT errno_t Open(const wchar_t* filename) NLIB_NOEXCEPT NLIB_NONNULL;
    NLIB_CHECK_RESULT errno_t FdOpen(nlib_fd fd) NLIB_NOEXCEPT;
    nlib_fd GetFd() const NLIB_NOEXCEPT { return fd_; }

 private:
    virtual size_t FillBuffer_(void* p, size_t nbytes) NLIB_NOEXCEPT NLIB_OVERRIDE;
    virtual bool Close_() NLIB_NOEXCEPT NLIB_OVERRIDE;
    virtual void* GetWorkBuffer_(size_t* nbytes) NLIB_NOEXCEPT NLIB_OVERRIDE;

 private:
    nlib_fd fd_;
    unsigned char* buf_;
    size_t buf_size_;
    bool is_internal_buf_;
    NLIB_DISALLOW_COPY_AND_ASSIGN(FileInputStream);
};

NLIB_NAMESPACE_END

#endif  // INCLUDE_NN_NLIB_FILEINPUTSTREAM_H_
