﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_FILEOUTPUTSTREAM_H_
#define INCLUDE_NN_NLIB_FILEOUTPUTSTREAM_H_

#include "nn/nlib/OutputStream.h"

NLIB_NAMESPACE_BEGIN

struct FileOutputStreamSettings {
    void* buf;
    size_t buffer_size;

 public:
    NLIB_CEXPR FileOutputStreamSettings() NLIB_NOEXCEPT : buf(NULL), buffer_size(4096) {}
    NLIB_CEXPR FileOutputStreamSettings(void* buf_, size_t buffer_size_) NLIB_NOEXCEPT
        : buf(buf_),
          buffer_size(buffer_size_) {}
};

class NLIB_VIS_PUBLIC FileOutputStream NLIB_FINAL : public OutputStream{
 public:
    FileOutputStream() NLIB_NOEXCEPT : fd_(NLIB_FD_INVALID),
                                       buf_(NULL),
                                       buf_size_(0),
                                       is_internal_buffer_(false) {}
    virtual ~FileOutputStream() NLIB_NOEXCEPT NLIB_FINAL NLIB_OVERRIDE;
    errno_t Init() NLIB_NOEXCEPT {
        NLIB_CEXPR FileOutputStreamSettings settings;
        return this->Init(settings);
    }
    NLIB_CHECK_RESULT errno_t Init(const FileOutputStreamSettings& settings) NLIB_NOEXCEPT;
    NLIB_CHECK_RESULT errno_t Open(const char* filename,
                                   int flags, int mode) NLIB_NOEXCEPT NLIB_NONNULL;
    NLIB_CHECK_RESULT errno_t Open(const char* filename) NLIB_NOEXCEPT {
        return Open(filename, NLIB_FD_O_WRONLY | NLIB_FD_O_CREAT | NLIB_FD_O_TRUNC, 0644);
    }
    NLIB_CHECK_RESULT errno_t Open(const char* filename, int flags) NLIB_NOEXCEPT {
        return Open(filename, flags, 0644);
    }
    NLIB_CHECK_RESULT errno_t Open(const wchar_t* filename,
                                   int flags, int mode) NLIB_NOEXCEPT NLIB_NONNULL;
    NLIB_CHECK_RESULT errno_t Open(const wchar_t* filename, int flags) NLIB_NOEXCEPT {
        return Open(filename, flags, 0644);
    }
    NLIB_CHECK_RESULT errno_t Open(const wchar_t* filename) NLIB_NOEXCEPT {
        return Open(filename, NLIB_FD_O_WRONLY | NLIB_FD_O_CREAT | NLIB_FD_O_TRUNC, 0644);
    }
    NLIB_CHECK_RESULT errno_t FdOpen(nlib_fd fd) NLIB_NOEXCEPT;
    nlib_fd GetFd() const NLIB_NOEXCEPT { return fd_; }

 private:
    virtual bool PushBuffer_(const void* p, size_t nbytes,
                             bool do_flush) NLIB_NOEXCEPT NLIB_OVERRIDE;
    virtual bool Close_() NLIB_NOEXCEPT NLIB_OVERRIDE;
    virtual bool WriteGather_(const nlib_fd_iovec* iov, int iovcnt) NLIB_NOEXCEPT NLIB_OVERRIDE;

 private:
    nlib_fd fd_;
    unsigned char* buf_;
    size_t buf_size_;
    bool is_internal_buffer_;
    NLIB_DISALLOW_COPY_AND_ASSIGN(FileOutputStream);
};

NLIB_NAMESPACE_END

#endif  // INCLUDE_NN_NLIB_FILEOUTPUTSTREAM_H_
