﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_FILESTYLEURI_H_
#define INCLUDE_NN_NLIB_FILESTYLEURI_H_

#include "nn/nlib/Config.h"
#include "nn/nlib/Uri.h"
#include "nn/nlib/Swap.h"

NLIB_NAMESPACE_BEGIN

class NLIB_VIS_PUBLIC FileStyleUri NLIB_FINAL {
 public:
    FileStyleUri() NLIB_NOEXCEPT : prv_(NULL) {}
    ~FileStyleUri() NLIB_NOEXCEPT;
    NLIB_MOVE_MEMBER_HELPER_1(FileStyleUri, prv_);
    bool FromUri(const Uri& uri) NLIB_NOEXCEPT;
    bool ToUri(Uri* uri) const NLIB_NOEXCEPT NLIB_NONNULL;
    bool Parse(const char* str) NLIB_NOEXCEPT NLIB_NONNULL;
    bool ComposeString(
        char* buf,
        size_t size) const NLIB_NOEXCEPT NLIB_NONNULL;
    template <size_t N>
    bool ComposeString(
        char (&buf)[N]) const NLIB_NOEXCEPT {
        return this->ComposeString(buf, N);
    }
    bool SetPath(const char* path) NLIB_NOEXCEPT;
    size_t GetNumSegment() const NLIB_NOEXCEPT;
    const char* GetSegment(size_t i) const NLIB_NOEXCEPT;
    bool AddBaseUri(const Uri& relative, const FileStyleUri& base) NLIB_NOEXCEPT;
    void Reset() NLIB_NOEXCEPT;
    bool IsPathDirectory() const NLIB_NOEXCEPT;
    void swap(FileStyleUri& rhs) NLIB_NOEXCEPT {
        using std::swap;
        swap(prv_, rhs.prv_);
    }

 private:
    NLIB_VIS_HIDDEN bool CheckInit() const NLIB_NOEXCEPT;
    struct FileStyleUriPrivate;
    mutable FileStyleUriPrivate* prv_;
    NLIB_DISALLOW_COPY_AND_ASSIGN(FileStyleUri);
};

NLIB_NAMESPACE_END
NLIB_DEFINE_STD_SWAP(::nlib_ns::FileStyleUri)
#endif  // INCLUDE_NN_NLIB_FILESTYLEURI_H_
