﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_NATIVEPATHMAPPER_H_
#define INCLUDE_NN_NLIB_NATIVEPATHMAPPER_H_

#include "nn/nlib/Config.h"
#include "nn/nlib/Swap.h"

NLIB_NAMESPACE_BEGIN

class NLIB_VIS_PUBLIC NativePathMapper {
 public:
    NativePathMapper() NLIB_NOEXCEPT : prv_(NULL) {}
    ~NativePathMapper() NLIB_NOEXCEPT;
    NLIB_MOVE_MEMBER_HELPER_1(NativePathMapper, prv_);
    errno_t AddMap(const char* native_prefix,
                   const char* uri_prefix) NLIB_NOEXCEPT;
    errno_t ResolvePath(
        size_t* count,
        char* native,
        size_t n,
        const char* uri_path) const NLIB_NOEXCEPT;
    template <size_t N>
    errno_t ResolvePath(
        size_t* count,
        char (&native)[N],
        const char* uri_path) const NLIB_NOEXCEPT {
        return ResolvePath(count, native, N, uri_path);
    }
    void swap(NativePathMapper& rhs) NLIB_NOEXCEPT {
        using std::swap;
        swap(prv_, rhs.prv_);
    }

 private:
    struct NativePathMapperPrivate;
    NativePathMapperPrivate* prv_;
    NLIB_DISALLOW_COPY_AND_ASSIGN(NativePathMapper);
};

NLIB_NAMESPACE_END
NLIB_DEFINE_STD_SWAP(::nlib_ns::NativePathMapper)
#endif  // INCLUDE_NN_NLIB_NATIVEPATHMAPPER_H_
