﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_OUTPUTCONVERTERSTREAM_H_
#define INCLUDE_NN_NLIB_OUTPUTCONVERTERSTREAM_H_

#include "nn/nlib/OutputStream.h"

NLIB_NAMESPACE_BEGIN

class NLIB_VIS_PUBLIC OutputTransform {
    // define a derived class and use it from OutputConverterStream
 public:
    virtual ~OutputTransform() {}
    virtual errno_t Transform(OutputStream* os, const void* p, size_t nBytes, bool doFlush) = 0;

    // called only once at first, and pass the workbuffer which OutputConverterStream uses.
    // The derived class has to manage the memory.
    virtual void* GetWorkBuffer(size_t* n) = 0;

    // called when OutputConverterStream::SetStream() is called.
    virtual errno_t OnSetStream(OutputStream* os) = 0;

    // called when the stream is closing.
    virtual errno_t OnClose(OutputStream* os) = 0;
};

class NLIB_VIS_PUBLIC OutputConverterStream : public OutputStream {
 public:
    OutputConverterStream() NLIB_NOEXCEPT;

    errno_t SetStream(OutputStream* ostr) NLIB_NOEXCEPT;
    OutputStream* GetStream() const NLIB_NOEXCEPT { return stream_; }

    // NOTE:
    // The destructor does not close its stream_, nor call OutputTransform::OnClose().
    // it is because OutputConverterStream does not own stream_ nor cvtr_.
    virtual ~OutputConverterStream() NLIB_NOEXCEPT NLIB_OVERRIDE {}

 protected:
    // Note that SetTransform returns ENOMEM when cvtr->GetWorkBuffer() fails
    errno_t SetTransform(OutputTransform* cvtr) NLIB_NOEXCEPT;

 private:
    virtual bool PushBuffer_(const void* p, size_t nbytes,
                             bool do_flush) NLIB_NOEXCEPT NLIB_OVERRIDE NLIB_FINAL;
    virtual bool Close_() NLIB_NOEXCEPT NLIB_OVERRIDE NLIB_FINAL;
    virtual void* GetWorkBuffer_(size_t* nbytes) NLIB_NOEXCEPT NLIB_OVERRIDE NLIB_FINAL;

 private:
    OutputStream* stream_;
    OutputTransform* cvtr_;
    size_t ocs_bufsize_;
    unsigned char* ocs_buf_;

    NLIB_DISALLOW_COPY_AND_ASSIGN(OutputConverterStream);
};

template <class TRANSFORM>
class OutputConverterStreamTempl : public OutputConverterStream {
 public:
    typedef TRANSFORM TransformType;

    OutputConverterStreamTempl() NLIB_NOEXCEPT {
        this->SetTransform(&m_Transform);
    }
    virtual ~OutputConverterStreamTempl() NLIB_NOEXCEPT NLIB_OVERRIDE {
        // note that OutputConverterStream::Close_() is final
        bool result = this->Close();
        NLIB_UNUSED(result);
    }

 private:
    void SetTransform(OutputTransform* cvtr) NLIB_NOEXCEPT {
        // to hide SetTransform()
        errno_t e = OutputConverterStream::SetTransform(cvtr);
        if (e != 0) this->SetError(e);
    }

 protected:
    TRANSFORM m_Transform;

 private:
    NLIB_DISALLOW_COPY_AND_ASSIGN(OutputConverterStreamTempl);
};

NLIB_NAMESPACE_END

#endif  // INCLUDE_NN_NLIB_OUTPUTCONVERTERSTREAM_H_
