﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_PLATFORM_ERRNO_H_
#define INCLUDE_NN_NLIB_PLATFORM_ERRNO_H_

#ifndef INCLUDE_NN_NLIB_PLATFORM_H_
#error Do not include this file directly
#endif

/*
  err=( \
    "E2BIG" "EACCES" "EADDRINUSE" "EADDRNOTAVAIL" "EAFNOSUPPORT" "EAGAIN" "EALREADY" "EBADF" \
    "EBADMSG" "EBUSY" "ECANCELED" "ECHILD" "ECONNABORTED" "ECONNREFUSED" "ECONNRESET" "EDEADLK" \
    "EDESTADDRREQ" "EDOM" "EDQUOT" "EEXIST" "EFAULT" "EFBIG" "EHOSTUNREACH" "EIDRM" "EILSEQ" \
    "EINPROGRESS" "EINTR" "EINVAL" "EIO" "EISCONN" "EISDIR" "ELOOP" "EMFILE" "EMLINK" "EMSGSIZE" \
    "EMULTIHOP" "ENAMETOOLONG" "ENETDOWN" "ENETRESET" "ENETUNREACH" "ENFILE" "ENOBUFS" "ENODATA" \
    "ENODEV" "ENOENT" "ENOEXEC" "ENOLCK" "ENOLINK" "ENOMEM" "ENOMSG" "ENOPROTOOPT" "ENOSPC" \
    "ENOSR" "ENOSYS" "ENOTCONN" "ENOTDIR" "ENOTEMPTY" "ENOTRECOVERABLE" "ENOTSOCK" "ENOTSUP" \
    "ENOTTY" "ENXIO" "EOPNOTSUPP" "EOVERFLOW" "EOWNERDEAD" "EPERM" "EPIPE" "EPROTO" \
    "EPROTONOSUPPORT" "EPROTOTYPE" "ERANGE" "EROFS" "ESPIPE" "ESRCH" "ESTALE" "ETIME" "ETIMEDOUT" \
    "ETXTBSY" "EWOULDBLOCK" "EXDEV" \
  )
  files=($(find . | grep errno))
  for e in ${err[@]}
  do
  grep "\s$e\s" ${files[@]} > /dev/null
  if [ $? -gt 0 ]; then
    echo "$e is missing"
  fi
  done
*/

#define NLIB_E_BASE 2000

// if boost/cerrno.hpp has definitions on errno values,
// nlib defines the same values which boost/cerrno.hpp defines.

// [E2BIG]
//     Argument list too long.
#ifndef E2BIG
# define E2BIG 9946
#endif

// [EACCES]
//    Permission denied.
#ifndef EACCES
# define EACCES 9973
#endif

// [EADDRINUSE]
//    Address in use.
#ifndef EADDRINUSE
# define EADDRINUSE 9902
#endif

// [EADDRNOTAVAIL]
//    Address not available.
#ifndef EADDRNOTAVAIL
# define EADDRNOTAVAIL 9903
#endif

// [EAFNOSUPPORT]
//    Address family not supported.
#ifndef EAFNOSUPPORT
# define EAFNOSUPPORT 9901
#endif

// [EAGAIN]
//    Resource unavailable, try again (may be the same value as [EWOULDBLOCK]).
#ifndef EAGAIN
# define EAGAIN 9976
#endif

// [EALREADY]
//    Connection already in progress.
#ifndef EALREADY
# define EALREADY 9907
#endif

// [EBADF]
//    Bad file descriptor.
#ifndef EBADF
# define EBADF 9949
#endif

// [EBADMSG]
//    Bad message.
#ifndef EBADMSG
# define EBADMSG 9905
#endif

// [EBUSY]
//    Device or resource busy.
#ifndef EBUSY
# define EBUSY 9952
#endif

// [ECANCELED]
// Operation canceled.
#ifndef ECANCELED
# define ECANCELED 9927
#endif

// [ECHILD]
//    No child processes.
#ifndef ECHILD
# define ECHILD 9963
#endif

// [ECONNABORTED]
//    Connection aborted.
#ifndef ECONNABORTED
# define ECONNABORTED 9906
#endif

// [ECONNREFUSED]
//    Connection refused.
#ifndef ECONNREFUSED
# define ECONNREFUSED 9908
#endif

// [ECONNRESET]
//    Connection reset.
#ifndef ECONNRESET
# define ECONNRESET 9909
#endif

// [EDEADLK]
//    Resource deadlock would occur.
#ifndef EDEADLK
# define EDEADLK 9975
#endif

// [EDESTADDRREQ]
//    Destination address required.
#ifndef EDESTADDRREQ
# define EDESTADDRREQ 9910
#endif

// [EDOM]
//    Mathematics argument out of domain of function.
#ifndef EDOM
# define EDOM 9947
#endif

// [EDQUOT]
//    Reserved.
#ifndef EDQUOT
# define EDQUOT (NLIB_E_BASE + 19)
#endif

// [EEXIST]
//    File exists.
#ifndef EEXIST
# define EEXIST 9955
#endif

// [EFAULT]
//    Bad address.
#ifndef EFAULT
# define EFAULT 9948
#endif

// [EFBIG]
//    File too large.
#ifndef EFBIG
# define EFBIG 9956
#endif

// [EHOSTUNREACH]
//    Host is unreachable.
#ifndef EHOSTUNREACH
# define EHOSTUNREACH 9911
#endif

// [EIDRM]
//    Identifier removed.
#ifndef EIDRM
# define EIDRM 9912
#endif

// [EILSEQ]
//    Illegal byte sequence.
#ifndef EILSEQ
# define EILSEQ 9945
#endif

// [EINPROGRESS]
//    Operation in progress.
#ifndef EINPROGRESS
# define EINPROGRESS 9928
#endif

// [EINTR]
//    Interrupted function.
#ifndef EINTR
# define EINTR 9959
#endif

// [EINVAL]
//    Invalid argument.
#ifndef EINVAL
# define EINVAL 9943
#endif

// [EIO]
//    I/O error.
#ifndef EIO
# define EIO 9961
#endif

// [EISCONN]
//    Socket is connected.
#ifndef EISCONN
# define EISCONN 9904
#endif

// [EISDIR]
//    Is a directory.
#ifndef EISDIR
# define EISDIR 9962
#endif

// [ELOOP]
//    Too many levels of symbolic links.
#ifndef ELOOP
# define ELOOP 9939
#endif

// [EMFILE]
//    File descriptor value too large.
#ifndef EMFILE
# define EMFILE 9978
#endif

// [EMLINK]
//    Too many links.
#ifndef EMLINK
# define EMLINK 9979
#endif

// [EMSGSIZE]
//    Message too large.
#ifndef EMSGSIZE
# define EMSGSIZE 9913
#endif

// [EMULTIHOP]
//    Reserved.
#ifndef EMULTIHOP
# define EMULTIHOP (NLIB_E_BASE + 36)
#endif

// [ENAMETOOLONG]
//    Filename too long.
#ifndef ENAMETOOLONG
# define ENAMETOOLONG 9957
#endif

// [ENETDOWN]
//    Network is down.
#ifndef ENETDOWN
# define ENETDOWN 9914
#endif

// [ENETRESET]
//    Connection aborted by network.
#ifndef ENETRESET
# define ENETRESET 9915
#endif

// [ENETUNREACH]
//    Network unreachable.
#ifndef ENETUNREACH
# define ENETUNREACH 9916
#endif

// [ENFILE]
//    Too many files open in system.
#ifndef ENFILE
# define ENFILE 9977
#endif

// [ENOBUFS]
//    No buffer space available.
#ifndef ENOBUFS
# define ENOBUFS 9917
#endif

// [ENODATA]
//    [OB XSR] [Option Start] No message is available on the STREAM head read queue. [Option End]
#ifndef ENODATA
# define ENODATA 9919
#endif

// [ENODEV]
//    No such device.
#ifndef ENODEV
# define ENODEV 9967
#endif

// [ENOENT]
//    No such file or directory.
#ifndef ENOENT
# define ENOENT 9968
#endif

// [ENOEXEC]
//    Executable file format error.
#ifndef ENOEXEC
# define ENOEXEC 9954
#endif

// [ENOLCK]
//    No locks available.
#ifndef ENOLCK
# define ENOLCK 9964
#endif

// [ENOLINK]
//    Reserved.
#ifndef ENOLINK
# define ENOLINK 9918
#endif

// [ENOMEM]
//    Not enough space.
#ifndef ENOMEM
# define ENOMEM 9971
#endif

// [ENOMSG]
//    No message of the desired type.
#ifndef ENOMSG
# define ENOMSG 9920
#endif

// [ENOPROTOOPT]
//    Protocol not available.
#ifndef ENOPROTOOPT
# define ENOPROTOOPT 9921
#endif

// [ENOSPC]
//    No space left on device.
#ifndef ENOSPC
# define ENOSPC 9965
#endif

// [ENOSR]
//    [OB XSR] [Option Start] No STREAM resources. [Option End]
#ifndef ENOSR
# define ENOSR 9922
#endif

// [ENOSTR]
//    [OB XSR] [Option Start] Not a STREAM. [Option End]
#ifndef ENOSTR
# define ENOSTR 9924
#endif

// [ENOSYS]
//    Function not supported.
#ifndef ENOSYS
# define ENOSYS 9942
#endif

// [ENOTCONN]
//    The socket is not connected.
#ifndef ENOTCONN
# define ENOTCONN 9925
#endif

// [ENOTDIR]
//    Not a directory or a symbolic link to a directory.
#ifndef ENOTDIR
# define ENOTDIR 9970
#endif

// [ENOTEMPTY]
//    Directory not empty.
#ifndef ENOTEMPTY
# define ENOTEMPTY 9953
#endif

// [ENOTRECOVERABLE]
//    State not recoverable.
#ifndef ENOTRECOVERABLE
# define ENOTRECOVERABLE 9934
#endif

// [ENOTSOCK]
//    Not a socket.
#ifndef ENOTSOCK
# define ENOTSOCK 9923
#endif

// [ENOTSUP]
//    Not supported (may be the same value as [EOPNOTSUPP]).
#ifndef ENOTSUP
# define ENOTSUP 9926
#endif

// [ENOTTY]
//    Inappropriate I/O control operation.
#ifndef ENOTTY
# define ENOTTY 9958
#endif

// [ENXIO]
//    No such device or address.
#ifndef ENXIO
# define ENXIO 9966
#endif

// [EOPNOTSUPP]
//    Operation not supported on socket (may be the same value as [ENOTSUP]).
#ifndef EOPNOTSUPP
# define EOPNOTSUPP 9929
#endif

// [EOVERFLOW]
//    Value too large to be stored in data type.
#ifndef EOVERFLOW
# define EOVERFLOW 9940
#endif

// [EOWNERDEAD]
//    Previous owner died.
#ifndef EOWNERDEAD
# define EOWNERDEAD  9931
#endif

// [EPERM]
//    Operation not permitted.
#ifndef EPERM
# define EPERM 9972
#endif

// [EPIPE]
//    Broken pipe.
#ifndef EPIPE
# define EPIPE 9950
#endif

// [EPROTO]
//    Protocol error.
#ifndef EPROTO
# define EPROTO 9932
#endif

// [EPROTONOSUPPORT]
//    Protocol not supported.
#ifndef EPROTONOSUPPORT
# define EPROTONOSUPPORT 9933
#endif

// [EPROTOTYPE]
//    Protocol wrong type for socket.
#ifndef EPROTOTYPE
# define EPROTOTYPE 9941
#endif

// [ERANGE]
//    Result too large.
#ifndef ERANGE
# define ERANGE 9944
#endif

// [EROFS]
//    Read-only file system.
#ifndef EROFS
# define EROFS 9974
#endif

// [ESPIPE]
//    Invalid seek.
#ifndef ESPIPE
# define ESPIPE 9960
#endif

// [ESRCH]
//    No such process.
#ifndef ESRCH
# define ESRCH 9969
#endif

// [ESTALE]
//    Reserved.
#ifndef ESTALE
# define ESTALE (NLIB_E_BASE + 75)
#endif

// [ETIME]
//    [OB XSR] [Option Start] Stream ioctl() timeout. [Option End]
#ifndef ETIME
# define ETIME 9935
#endif

// [ETIMEDOUT]
//    Connection timed out.
#ifndef ETIMEDOUT
# define ETIMEDOUT 9938
#endif

// [ETXTBSY]
//    Text file busy.
#ifndef ETXTBSY
# define ETXTBSY 9936
#endif

// [EWOULDBLOCK]
//    Operation would block (may be the same value as [EAGAIN]).
#ifndef EWOULDBLOCK
# define EWOULDBLOCK 9930
#endif

// [EXDEV]
//    Cross-device link.
#ifndef EXDEV
# define EXDEV 9951
#endif

#endif  // INCLUDE_NN_NLIB_PLATFORM_ERRNO_H_
