﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_PLATFORM_UNIX_H_
#define INCLUDE_NN_NLIB_PLATFORM_UNIX_H_
#ifndef INCLUDE_NN_NLIB_PLATFORM_H_
# error do not include directly
#endif

#if defined(__linux__)   || \
    defined(__FreeBSD__) || \
    defined(__CYGWIN__)  || \
    (defined(__APPLE__) && defined(__MACH__))

#ifdef __cplusplus
// http://cpprefjp.github.io/reference/cstdint.html
// Older libc may require __STDC_LIMIT_MACROS and __STDC_CONSTANT_MACROS defined.
#ifndef __STDC_LIMIT_MACROS
#warning __STDC_LIMIT_MACROS not defined, compile may fail
#define __STDC_LIMIT_MACROS
#endif
#ifndef __STDC_CONSTANT_MACROS
#warning __STDC_CONSTANT_MACROS not defined, compile may fail
#define __STDC_CONSTANT_MACROS
#endif
#endif

#if defined(__APPLE__) && defined(__MACH__)
#define _DARWIN_UNLIMITED_SELECT
#include <libkern/OSAtomic.h>
#include <errno.h>
#if __has_include( <os/lock.h> )
#include <os/lock.h>
#endif
#endif

#ifdef __cplusplus
extern "C" {
#endif

#ifndef NLIB_UNIX
# define NLIB_UNIX
#endif

// For now, only supports gcc or clang
#if !defined(__GNUC__) && !defined(__clang__)
# error
#endif

#if !defined(__FreeBSD__) && !defined(__APPLE__)
// checking __GNU_LIBRARY__, __GLIBC__, __GLIBC_MINOR__ to detect glibc
#include <features.h>
#endif
#include <pthread.h>  // for PTHREAD_MUTEX_INITIALIZER, ....
#include <semaphore.h>  // for sem_t
#include <sys/types.h>  // for pthread_mutex_t, ....
#include <sys/uio.h>  // struct iovec
#include <fcntl.h>  // NOLINT
#include <sys/socket.h>
#include <netinet/tcp.h>  // TCP_FASTOPEN
#include <poll.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <stdint.h>

#if defined(__FreeBSD__) || defined(__APPLE__)
#include <dispatch/dispatch.h>
#endif

#if defined(__i386__) || defined(__x86_64__)
# include <x86intrin.h>
#endif

#ifndef __CYGWIN__
# define NLIB_VIS_HIDDEN __attribute__((visibility("hidden")))
# define NLIB_VIS_PUBLIC __attribute__((visibility("default")))
# define NLIB_WEAKSYMBOL __attribute__((weak))
#else
# define NLIB_VIS_HIDDEN
# define NLIB_VIS_PUBLIC
# define NLIB_WEAKSYMBOL
#endif

#define NLIB_ALWAYS_INLINE inline __attribute__((always_inline))
#define NLIB_NEVER_INLINE __attribute__((__noinline__))
#define NLIB_LIKELY(x) __builtin_expect(!!(x), 1)
#define NLIB_UNLIKELY(x) __builtin_expect(!!(x), 0)
#define NLIB_EXPECT(var, exp_value) __builtin_expect((var), (exp_value))
#define NLIB_CHECK_RESULT __attribute__((warn_unused_result))
#define NLIB_NORETURN __attribute__((noreturn))
#define NLIB_NONNULL __attribute__((nonnull))
#define NLIB_NONNULL_1 __attribute__((nonnull (1)))
#define NLIB_NONNULL_2 __attribute__((nonnull (2)))
#define NLIB_NONNULL_3 __attribute__((nonnull (3)))
#define NLIB_NONNULL_4 __attribute__((nonnull (4)))
#define NLIB_NONNULL_5 __attribute__((nonnull (5)))
#define NLIB_NONNULL_ENABLED
#define NLIB_ATTRIBUTE_MALLOC __attribute__((malloc))
#define NLIB_ATTRIBUTE_PURE __attribute__((pure))
#define NLIB_ATTRIBUTE_CONST __attribute__((const))

#ifdef __clang__
# if __has_attribute(alloc_size)
#  define NLIB_ATTRIBUTE_ALLOC_SIZE1(n) __attribute__((alloc_size(n)))
#  define NLIB_ATTRIBUTE_ALLOC_SIZE2(n0, n1) __attribute__((alloc_size(n0, n1)))
# else
#  define NLIB_ATTRIBUTE_ALLOC_SIZE1(n)
#  define NLIB_ATTRIBUTE_ALLOC_SIZE2(n0, n1)
# endif
# if __has_attribute(alloc_align)
#  define NLIB_ATTRIBUTE_ALLOC_ALIGN(algn) __attribute__((alloc_align(algn)))
# else
#  define NLIB_ATTRIBUTE_ALLOC_ALIGN(algn)
# endif
# if __has_attribute(assume_aligned)
#  define NLIB_ATTRIBUTE_ASSUME_ALIGNED(n) __attribute__((assume_aligned(n)))
# else
#  define NLIB_ATTRIBUTE_ASSUME_ALIGNED(n)
# endif
#else
# define NLIB_ATTRIBUTE_ALLOC_SIZE1(n) __attribute__((alloc_size(n)))
# define NLIB_ATTRIBUTE_ALLOC_SIZE2(n0, n1) __attribute__((alloc_size(n0, n1)))
# if __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 9)
#  define NLIB_ATTRIBUTE_ALLOC_ALIGN(algn) __attribute__((alloc_align(algn)))
#  define NLIB_ATTRIBUTE_ASSUME_ALIGNED(n) __attribute__((assume_aligned(n)))
# else
#  define NLIB_ATTRIBUTE_ALLOC_ALIGN(algn)
#  define NLIB_ATTRIBUTE_ASSUME_ALIGNED(n)
# endif
#endif

#ifndef NLIB_DEPRECATED
#define NLIB_DEPRECATED __attribute__((deprecated))
#endif
#ifndef NLIB_DEPRECATED_MSG
#define NLIB_DEPRECATED_MSG(msg) __attribute__((deprecated))
#endif

#if defined(__LITTLE_ENDIAN__) || defined(__LITTLE_ENDIAN)
# define NLIB_LITTLE_ENDIAN
#elif defined(__BIG_ENDIAN__) || defined(__BIG_ENDIAN)
# undef NLIB_LITTLE_ENDIAN
#else
# if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
#  define NLIB_LITTLE_ENDIAN
# else
#  undef NLIB_LITTLE_ENDIAN
# endif
#endif
#if defined(__x86_64__) || defined(__aarch64__)
# define NLIB_64BIT
#endif

#if !defined(__clang__) && defined(__GNUC__) && __GNUC__ == 4 && __GNUC_MINOR__ < 7
# if !defined(__i386__) && !defined(__x86_64__)
#  error Sorry
# endif
#define NLIB_MEMORY_ORDER_RELEASE __asm__ __volatile__("sfence": : :"memory")
#define NLIB_MEMORY_ORDER_ACQUIRE __asm__ __volatile__("lfence": : :"memory")
#define NLIB_MEMORY_ORDER_ACQ_REL __asm__ __volatile__("mfence": : :"memory")
#define NLIB_MEMORY_ORDER_SEQ_CST __sync_synchronize()
#else
#define NLIB_MEMORY_ORDER_RELEASE __atomic_thread_fence(__ATOMIC_RELEASE)
#define NLIB_MEMORY_ORDER_ACQUIRE __atomic_thread_fence(__ATOMIC_ACQUIRE)
#define NLIB_MEMORY_ORDER_ACQ_REL __atomic_thread_fence(__ATOMIC_ACQ_REL)
#define NLIB_MEMORY_ORDER_SEQ_CST __atomic_thread_fence(__ATOMIC_SEQ_CST)
#endif

typedef pthread_key_t nlib_tls;
#define NLIB_PTHREAD_nlib_tls_alloc
#define NLIB_PTHREAD_nlib_tls_free
#define NLIB_PTHREAD_nlib_tls_setvalue
#define NLIB_PTHREAD_nlib_tls_getvalue

#ifndef _LIBCPP_VERSION
NLIB_CAPABILITY("mutex")
#endif
typedef pthread_mutex_t nlib_mutex;

#ifdef PTHREAD_ADAPTIVE_MUTEX_INITIALIZER_NP
# ifdef __FreeBSD__
// https://stackoverflow.com/questions/10369606/constexpr-pointer-value
// ((pthread_mutex_t)1) in FreeBSD
#  define NLIB_MUTEX_INITIALIZER (__builtin_constant_p(PTHREAD_ADAPTIVE_MUTEX_INITIALIZER_NP) ? \
                                  PTHREAD_ADAPTIVE_MUTEX_INITIALIZER_NP :                       \
                                  PTHREAD_ADAPTIVE_MUTEX_INITIALIZER_NP)
# else
#  define NLIB_MUTEX_INITIALIZER PTHREAD_ADAPTIVE_MUTEX_INITIALIZER_NP
# endif
#else
# define NLIB_PTHREAD_nlib_mutex_init
# define NLIB_MUTEX_INITIALIZER PTHREAD_MUTEX_INITIALIZER
#endif

#ifndef __FreeBSD__
#define NLIB_PTHREAD_nlib_mutex_lock
#define NLIB_PTHREAD_nlib_mutex_unlock
#define NLIB_PTHREAD_nlib_mutex_trylock
#define NLIB_PTHREAD_nlib_mutex_destroy
#endif

#if defined(PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP)
# define NLIB_RECURSIVE_MUTEX_INITIALIZER PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP
# define NLIB_RECURSIVE_TIMED_MUTEX_INITIALIZER PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP
#elif defined(PTHREAD_RECURSIVE_MUTEX_INITIALIZER)
# define NLIB_RECURSIVE_MUTEX_INITIALIZER PTHREAD_RECURSIVE_MUTEX_INITIALIZER
# define NLIB_RECURSIVE_TIMED_MUTEX_INITIALIZER PTHREAD_RECURSIVE_MUTEX_INITIALIZER
#elif defined(__FreeBSD__)
// FreeBSD does not support static initializer for recursive mutex
// nlib initializes recursive mutex if the value is 255
# define NLIB_RECURSIVE_MUTEX_INITIALIZER (__builtin_constant_p((pthread_mutex_t)255) ? \
                                           (pthread_mutex_t)255 : (pthread_mutex_t)255)
# define NLIB_RECURSIVE_TIMED_MUTEX_INITIALIZER (__builtin_constant_p((pthread_mutex_t)255) ? \
                                                 (pthread_mutex_t)255 : (pthread_mutex_t)255)
#elif defined(NLIB_ALPINE)
// hacked...
#define NLIB_RECURSIVE_MUTEX_INITIALIZER {{{1}}}
#define NLIB_RECURSIVE_TIMED_MUTEX_INITIALIZER {{{1}}}
#else
# error Sorry
#endif

#if defined(__APPLE__)
typedef int nlib_semaphore;
#else
typedef sem_t nlib_semaphore;
#endif

typedef pthread_cond_t nlib_cond;
#define NLIB_COND_INITIALIZER PTHREAD_COND_INITIALIZER

#define NLIB_PTHREAD_nlib_cond_init
#define NLIB_PTHREAD_nlib_cond_signal
#define NLIB_PTHREAD_nlib_cond_broadcast
#define NLIB_PTHREAD_nlib_cond_wait
#define NLIB_PTHREAD_nlib_cond_destroy

typedef pthread_t nlib_thread;

#define NLIB_PTHREAD_nlib_thread_join
#define NLIB_PTHREAD_nlib_thread_detach
#define NLIB_PTHREAD_nlib_thread_equal
#define NLIB_PTHREAD_nlib_thread_self

#if defined(__APPLE__)
#define NLIB_SPINLOCK_HAS_NATIVE
#if __has_include( <os/lock.h> )
typedef os_unfair_lock nlib_spinlock;
#define NLIB_SPINLOCK_INITIALIZER OS_UNFAIR_LOCK_INIT
static NLIB_ALWAYS_INLINE void nlib_spinlock_init(nlib_spinlock* lock) {
    *lock = OS_UNFAIR_LOCK_INIT;
}
static NLIB_ALWAYS_INLINE void nlib_spinlock_lock(nlib_spinlock* lock) {
    os_unfair_lock_lock(lock);
}
static NLIB_ALWAYS_INLINE int nlib_spinlock_trylock(nlib_spinlock* lock) {
    return os_unfair_lock_trylock(lock) ? 0 : EBUSY;
}
static NLIB_ALWAYS_INLINE void nlib_spinlock_unlock(nlib_spinlock* lock) {
    os_unfair_lock_unlock(lock);
}
#else
typedef OSSpinLock nlib_spinlock;
#define NLIB_SPINLOCK_INITIALIZER (0)
static NLIB_ALWAYS_INLINE void nlib_spinlock_init(nlib_spinlock* lock) {
    *lock = 0;
}
static NLIB_ALWAYS_INLINE void nlib_spinlock_lock(nlib_spinlock* lock) {
    OSSpinLockLock(lock);
}
static NLIB_ALWAYS_INLINE int nlib_spinlock_trylock(nlib_spinlock* lock) {
    return OSSpinLockTry(lock) ? 0 : EBUSY;
}
static NLIB_ALWAYS_INLINE void nlib_spinlock_unlock(nlib_spinlock* lock) {
    OSSpinLockUnlock(lock);
}
#endif
#endif

#ifdef __cplusplus
}
#endif

#if defined(__clang__)
# if __has_feature(cxx_unicode_literals)
#  define NLIB_CXX11_NEW_CHARACTER_TYPES
# endif
# if __has_feature(cxx_exceptions)
#  if __has_feature(cxx_noexcept)
#   define NLIB_CXX11_NOEXCEPT
#  endif
# else
#  define NLIB_NOEXCEPT
# endif
#else
// __GNUC__
# if __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 4)
#  define NLIB_CXX11_NEW_CHARACTER_TYPES
# endif
# if __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6)
#   define NLIB_CXX11_NOEXCEPT
# endif
// # if !defined(__CYGWIN__) && (!defined(__GLIBC__) || __GLIBC__ < 2 || __GLIBC_MINOR__ < 1)
// #  error Sorry, glibc is old.
// # endif
#endif

#define NLIB_ONCE_HAS_NATIVE
#define NLIB_TIMESPEC_HAS_NATIVE
#define NLIB_IOVEC_HAS_NATIVE

#ifdef PTHREAD_RWLOCK_INITIALIZER
# define NLIB_RWLOCK_HAS_NATIVE
#endif
#ifdef NLIB_RWLOCK_HAS_NATIVE
#ifndef _LIBCPP_VERSION
NLIB_CAPABILITY("mutex")
#endif
typedef pthread_rwlock_t nlib_rwlock;
#define NLIB_RWLOCK_INITIALIZER PTHREAD_RWLOCK_INITIALIZER

#define NLIB_PTHREAD_nlib_rwlock_init
#define NLIB_PTHREAD_nlib_rwlock_destroy
#define NLIB_PTHREAD_nlib_rwlock_tryrdlock
#define NLIB_PTHREAD_nlib_rwlock_trywrlock
#define NLIB_PTHREAD_nlib_rwlock_rdlock
#define NLIB_PTHREAD_nlib_rwlock_rdunlock
#define NLIB_PTHREAD_nlib_rwlock_wrlock
#define NLIB_PTHREAD_nlib_rwlock_wrunlock
#endif

#ifdef PTHREAD_BARRIER_SERIAL_THREAD
# define NLIB_BARRIER_HAS_NATIVE
#endif
#ifdef NLIB_BARRIER_HAS_NATIVE
typedef pthread_barrier_t nlib_barrier;
#define NLIB_PTHREAD_nlib_barrier_init
#define NLIB_PTHREAD_nlib_barrier_destroy
#endif

#define NLIB_THREAD_ATTR_HAS_NATIVE

#ifndef pthread_cleanup_push
# error pthread_cleanup_push must be a macro
#endif

#ifndef pthread_cleanup_pop
# error pthread_cleanup_pop must be a macro
#endif

#ifdef __linux__
// NOTE:
// The implementation by timerfd_create() and epoll() consumes file descriptors.
// The difference of the maximum numbers of file descriptors might limit the portability.
// Also, the performance is not better than our generic version.
// #define NLIB_TIMER_HAS_NATIVE
#endif

#define NLIB_LIBC_nlib_memcmp
#define NLIB_LIBC_nlib_strlen
#define NLIB_LIBC_nlib_strnlen
#if defined(__STDC_LIB_EXT1__)
# define NLIB_LIBC_nlib_wcslen
# define NLIB_LIBC_nlib_wcsnlen
# define NLIB_LIBC_nlib_strncpy
# define NLIB_LIBC_nlib_strcpy
# define NLIB_LIBC_nlib_wcsncpy
# define NLIB_LIBC_nlib_wcscpy
#endif
#define NLIB_LIBC_nlib_strchr
#define NLIB_LIBC_nlib_strrchr

#ifdef __cplusplus
extern "C" {
#endif

#if (defined(__clang__) && defined(NLIB_64BIT)) || \
    (defined(__GNUC__) && __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 7))

#define NLIB_ATOMIC_RELAXED __ATOMIC_RELAXED
#define NLIB_ATOMIC_ACQUIRE __ATOMIC_ACQUIRE
#define NLIB_ATOMIC_RELEASE __ATOMIC_RELEASE
#define NLIB_ATOMIC_ACQ_REL __ATOMIC_ACQ_REL
#define NLIB_ATOMIC_SEQ_CST __ATOMIC_SEQ_CST

#if defined(NLIB_DOXYGEN)
int32_t nlib_atomic_load32(const int32_t* ptr, int memorder);
void nlib_atomic_store32(int32_t* ptr, int32_t val, int memorder);
int32_t nlib_atomic_exchange32(int32_t* ptr, int32_t val, int memorder);
int nlib_atomic_compare_exchange32(int32_t* ptr, int32_t* expected,
                                   int32_t desired, int weak,
                                   int success_memorder, int failure_memorder);
int32_t nlib_atomic_add_fetch32(int32_t* ptr, int32_t val, int memorder);
int32_t nlib_atomic_sub_fetch32(int32_t* ptr, int32_t val, int memorder);
int32_t nlib_atomic_and_fetch32(int32_t* ptr, int32_t val, int memorder);
int32_t nlib_atomic_xor_fetch32(int32_t* ptr, int32_t val, int memorder);
int32_t nlib_atomic_or_fetch32(int32_t* ptr, int32_t val, int memorder);
int32_t nlib_atomic_fetch_add32(int32_t* ptr, int32_t val, int memorder);
int32_t nlib_atomic_fetch_sub32(int32_t* ptr, int32_t val, int memorder);
int32_t nlib_atomic_fetch_and32(int32_t* ptr, int32_t val, int memorder);
int32_t nlib_atomic_fetch_xor32(int32_t* ptr, int32_t val, int memorder);
int32_t nlib_atomic_fetch_or32(int32_t* ptr, int32_t val, int memorder);

int64_t nlib_atomic_load64(const int64_t* ptr, int memorder);
void nlib_atomic_store64(int64_t* ptr, int64_t val, int memorder);
int64_t nlib_atomic_exchange64(int64_t* ptr, int64_t val, int memorder);
int nlib_atomic_compare_exchange64(int64_t* ptr, int64_t* expected,
                                   int64_t desired, int weak,
                                   int success_memorder, int failure_memorder);
int64_t nlib_atomic_add_fetch64(int64_t* ptr, int64_t val, int memorder);
int64_t nlib_atomic_sub_fetch64(int64_t* ptr, int64_t val, int memorder);
int64_t nlib_atomic_and_fetch64(int64_t* ptr, int64_t val, int memorder);
int64_t nlib_atomic_xor_fetch64(int64_t* ptr, int64_t val, int memorder);
int64_t nlib_atomic_or_fetch64(int64_t* ptr, int64_t val, int memorder);
int64_t nlib_atomic_fetch_add64(int64_t* ptr, int64_t val, int memorder);
int64_t nlib_atomic_fetch_sub64(int64_t* ptr, int64_t val, int memorder);
int64_t nlib_atomic_fetch_and64(int64_t* ptr, int64_t val, int memorder);
int64_t nlib_atomic_fetch_xor64(int64_t* ptr, int64_t val, int memorder);
int64_t nlib_atomic_fetch_or64(int64_t* ptr, int64_t val, int memorder);

void* nlib_atomic_loadptr(void* const* ptr, int memorder);
void nlib_atomic_storeptr(void** ptr, void* val, int memorder);
void* nlib_atomic_exchangeptr(void** ptr, void* val, int memorder);
int nlib_atomic_compare_exchangeptr(void** ptr, void** expected,
                                    void* desired, int weak,
                                    int success_memorder, int failure_memorder);
void nlib_atomic_thread_fence(int memorder);
#endif

NLIB_VIS_PUBLIC extern pthread_mutex_t nlib_tsan_lock;
#if defined(__has_feature) && __has_feature(thread_sanitizer)
#define NLIB_TSAN_LOCK pthread_mutex_lock(&nlib_tsan_lock);
#define NLIB_TSAN_UNLOCK pthread_mutex_unlock(&nlib_tsan_lock);
#else
#define NLIB_TSAN_LOCK
#define NLIB_TSAN_UNLOCK
#endif

static __inline int32_t nlib_atomic_load32(const int32_t* ptr, int memorder) {
    int32_t rval;
    NLIB_TSAN_LOCK
    rval = __atomic_load_n(ptr, memorder);
    NLIB_TSAN_UNLOCK
    return rval;
}

static __inline void nlib_atomic_store32(int32_t* ptr, int32_t val, int memorder) {
    NLIB_TSAN_LOCK
    __atomic_store_n(ptr, val, memorder);
    NLIB_TSAN_UNLOCK
}

static __inline int32_t nlib_atomic_exchange32(int32_t* ptr, int32_t val,
                                               int memorder) {
    int32_t rval;
    NLIB_TSAN_LOCK
    rval = __atomic_exchange_n(ptr, val, memorder);
    NLIB_TSAN_UNLOCK
    return rval;
}

static __inline int nlib_atomic_compare_exchange32(int32_t* ptr, int32_t* expected,
                                                   int32_t desired, int weak,
                                                   int success_memorder, int failure_memorder) {
    int rval;
    NLIB_TSAN_LOCK
    rval = __atomic_compare_exchange_n(ptr, expected, desired, weak,
                                       success_memorder, failure_memorder);
    NLIB_TSAN_UNLOCK
    return rval;
}

static __inline int32_t nlib_atomic_add_fetch32(int32_t* ptr, int32_t val,
                                                int memorder) {
    int32_t rval;
    NLIB_TSAN_LOCK
    rval = __atomic_add_fetch(ptr, val, memorder);
    NLIB_TSAN_UNLOCK
    return rval;
}

static __inline int32_t nlib_atomic_sub_fetch32(int32_t* ptr, int32_t val,
                                                int memorder) {
    int32_t rval;
    NLIB_TSAN_LOCK
    rval = __atomic_sub_fetch(ptr, val, memorder);
    NLIB_TSAN_UNLOCK
    return rval;
}

static __inline int32_t nlib_atomic_and_fetch32(int32_t* ptr, int32_t val,
                                                int memorder) {
    int32_t rval;
    NLIB_TSAN_LOCK
    rval = __atomic_and_fetch(ptr, val, memorder);
    NLIB_TSAN_UNLOCK
    return rval;
}

static __inline int32_t nlib_atomic_xor_fetch32(int32_t* ptr, int32_t val,
                                                int memorder) {
    int32_t rval;
    NLIB_TSAN_LOCK
    rval = __atomic_xor_fetch(ptr, val, memorder);
    NLIB_TSAN_UNLOCK
    return rval;
}

static __inline int32_t nlib_atomic_or_fetch32(int32_t* ptr, int32_t val,
                                               int memorder) {
    int32_t rval;
    NLIB_TSAN_LOCK
    rval = __atomic_or_fetch(ptr, val, memorder);
    NLIB_TSAN_UNLOCK
    return rval;
}

static __inline int32_t nlib_atomic_fetch_add32(int32_t* ptr, int32_t val,
                                                int memorder) {
    int32_t rval;
    NLIB_TSAN_LOCK
    rval = __atomic_fetch_add(ptr, val, memorder);
    NLIB_TSAN_UNLOCK
    return rval;
}

static __inline int32_t nlib_atomic_fetch_sub32(int32_t* ptr, int32_t val,
                                                int memorder) {
    int32_t rval;
    NLIB_TSAN_LOCK
    rval = __atomic_fetch_sub(ptr, val, memorder);
    NLIB_TSAN_UNLOCK
    return rval;
}

static __inline int32_t nlib_atomic_fetch_and32(int32_t* ptr, int32_t val,
                                                int memorder) {
    int32_t rval;
    NLIB_TSAN_LOCK
    rval = __atomic_fetch_and(ptr, val, memorder);
    NLIB_TSAN_UNLOCK
    return rval;
}

static __inline int32_t nlib_atomic_fetch_xor32(int32_t* ptr, int32_t val,
                                                int memorder) {
    int32_t rval;
    NLIB_TSAN_LOCK
    rval = __atomic_fetch_xor(ptr, val, memorder);
    NLIB_TSAN_UNLOCK
    return rval;
}

static __inline int32_t nlib_atomic_fetch_or32(int32_t* ptr, int32_t val,
                                               int memorder) {
    int32_t rval;
    NLIB_TSAN_LOCK
    rval = __atomic_fetch_or(ptr, val, memorder);
    NLIB_TSAN_UNLOCK
    return rval;
}

static __inline int64_t nlib_atomic_load64(const int64_t* ptr, int memorder) {
    int64_t rval;
    NLIB_TSAN_LOCK
    rval = __atomic_load_n(ptr, memorder);
    NLIB_TSAN_UNLOCK
    return rval;
}

static __inline void nlib_atomic_store64(int64_t* ptr, int64_t val, int memorder) {
    NLIB_TSAN_LOCK
    __atomic_store_n(ptr, val, memorder);
    NLIB_TSAN_UNLOCK
}

static __inline int64_t nlib_atomic_exchange64(int64_t* ptr, int64_t val,
                                               int memorder) {
    int64_t rval;
    NLIB_TSAN_LOCK
    rval = __atomic_exchange_n(ptr, val, memorder);
    NLIB_TSAN_UNLOCK
    return rval;
}

static __inline void* nlib_atomic_exchangeptr(void** ptr, void* val, int memorder) {
    void* rval;
    NLIB_TSAN_LOCK
    rval = __atomic_exchange_n(ptr, val, memorder);
    NLIB_TSAN_UNLOCK
    return rval;
}

static __inline int nlib_atomic_compare_exchange64(int64_t* ptr, int64_t* expected,
                                                   int64_t desired, int weak,
                                                   int success_memorder, int failure_memorder) {
    int rval;
    NLIB_TSAN_LOCK
    rval = __atomic_compare_exchange_n(ptr, expected, desired, weak,
                                       success_memorder, failure_memorder);
    NLIB_TSAN_UNLOCK
    return rval;
}

static __inline int64_t nlib_atomic_add_fetch64(int64_t* ptr, int64_t val,
                                                int memorder) {
    int64_t rval;
    NLIB_TSAN_LOCK
    rval = __atomic_add_fetch(ptr, val, memorder);
    NLIB_TSAN_UNLOCK
    return rval;
}

static __inline int64_t nlib_atomic_sub_fetch64(int64_t* ptr, int64_t val,
                                                int memorder) {
    int64_t rval;
    NLIB_TSAN_LOCK
    rval = __atomic_sub_fetch(ptr, val, memorder);
    NLIB_TSAN_UNLOCK
    return rval;
}

static __inline int64_t nlib_atomic_and_fetch64(int64_t* ptr, int64_t val,
                                                int memorder) {
    int64_t rval;
    NLIB_TSAN_LOCK
    rval = __atomic_and_fetch(ptr, val, memorder);
    NLIB_TSAN_UNLOCK
    return rval;
}

static __inline int64_t nlib_atomic_xor_fetch64(int64_t* ptr, int64_t val,
                                                int memorder) {
    int64_t rval;
    NLIB_TSAN_LOCK
    rval = __atomic_xor_fetch(ptr, val, memorder);
    NLIB_TSAN_UNLOCK
    return rval;
}

static __inline int64_t nlib_atomic_or_fetch64(int64_t* ptr, int64_t val,
                                               int memorder) {
    int64_t rval;
    NLIB_TSAN_LOCK
    rval = __atomic_or_fetch(ptr, val, memorder);
    NLIB_TSAN_UNLOCK
    return rval;
}

static __inline int64_t nlib_atomic_fetch_add64(int64_t* ptr, int64_t val,
                                                int memorder) {
    int64_t rval;
    NLIB_TSAN_LOCK
    rval = __atomic_fetch_add(ptr, val, memorder);
    NLIB_TSAN_UNLOCK
    return rval;
}

static __inline int64_t nlib_atomic_fetch_sub64(int64_t* ptr, int64_t val,
                                                int memorder) {
    int64_t rval;
    NLIB_TSAN_LOCK
    rval = __atomic_fetch_sub(ptr, val, memorder);
    NLIB_TSAN_UNLOCK
    return rval;
}

static __inline int64_t nlib_atomic_fetch_and64(int64_t* ptr, int64_t val,
                                                int memorder) {
    int64_t rval;
    NLIB_TSAN_LOCK
    rval = __atomic_fetch_and(ptr, val, memorder);
    NLIB_TSAN_UNLOCK
    return rval;
}

static __inline int64_t nlib_atomic_fetch_xor64(int64_t* ptr, int64_t val,
                                                int memorder) {
    int64_t rval;
    NLIB_TSAN_LOCK
    rval = __atomic_fetch_xor(ptr, val, memorder);
    NLIB_TSAN_UNLOCK
    return rval;
}

static __inline int64_t nlib_atomic_fetch_or64(int64_t* ptr, int64_t val,
                                               int memorder) {
    int64_t rval;
    NLIB_TSAN_LOCK
    rval = __atomic_fetch_or(ptr, val, memorder);
    NLIB_TSAN_UNLOCK
    return rval;
}

static __inline void* nlib_atomic_loadptr(void* const* ptr, int memorder) {
    void* rval;
    NLIB_TSAN_LOCK
    rval = __atomic_load_n(ptr, memorder);
    NLIB_TSAN_UNLOCK
    return rval;
}

static __inline void nlib_atomic_storeptr(void** ptr, void* val, int memorder) {
    NLIB_TSAN_LOCK
    __atomic_store_n(ptr, val, memorder);
    NLIB_TSAN_UNLOCK
}

static __inline int nlib_atomic_compare_exchangeptr(void** ptr, void** expected,
                                                    void* desired, int weak,
                                                    int success_memorder, int failure_memorder) {
    int rval;
    NLIB_TSAN_LOCK
    rval = __atomic_compare_exchange_n(ptr, expected, desired, weak,
                                       success_memorder, failure_memorder);
    NLIB_TSAN_UNLOCK
    return rval;
}

static __inline void nlib_atomic_thread_fence(int memorder) {
    __atomic_thread_fence(memorder);
}

#else
#define NLIB_ATOMIC_RELAXED 0
#define NLIB_ATOMIC_ACQUIRE 1
#define NLIB_ATOMIC_RELEASE 2
#define NLIB_ATOMIC_ACQ_REL 3
#define NLIB_ATOMIC_SEQ_CST 7

static __inline int32_t nlib_atomic_load32(const int32_t* ptr, int memorder) {
    int32_t rval = *(volatile int32_t*)ptr;
    (void)memorder;
#if !defined(__i386__) && !defined(__x86_64__)
    if (memorder & NLIB_ATOMIC_ACQUIRE)
        NLIB_MEMORY_ORDER_ACQUIRE;
#endif
    return rval;
}

static NLIB_ALWAYS_INLINE void nlib_atomic_store32(int32_t* ptr, int32_t val,
                                                   int memorder) {
    if (memorder == NLIB_ATOMIC_SEQ_CST)
        __sync_synchronize();
    else if (memorder & NLIB_ATOMIC_RELEASE)
        NLIB_MEMORY_ORDER_RELEASE;
    __sync_lock_test_and_set(ptr, val);
}

static __inline int32_t nlib_atomic_exchange32(int32_t* ptr, int32_t val,
                                               int memorder) {
    if (memorder == NLIB_ATOMIC_SEQ_CST)
        __sync_synchronize();
    else if (memorder & NLIB_ATOMIC_RELEASE)
        NLIB_MEMORY_ORDER_RELEASE;
    return __sync_lock_test_and_set(ptr, val);
}

static __inline int nlib_atomic_compare_exchange32(int32_t* ptr, int32_t* expected,
                                                   int32_t desired, int weak,
                                                   int success_memorder, int failure_memorder) {
    int32_t old = __sync_val_compare_and_swap(ptr, *expected, desired);
    if (old == *expected) return 1;
    *expected = old;

    (void)weak;
    (void)success_memorder;
    (void)failure_memorder;
    return 0;
}

static __inline int32_t nlib_atomic_add_fetch32(int32_t* ptr, int32_t val,
                                                int memorder) {
    (void)memorder;
    return __sync_add_and_fetch(ptr, val);
}

static __inline int32_t nlib_atomic_sub_fetch32(int32_t* ptr, int32_t val,
                                                int memorder) {
    (void)memorder;
    return __sync_sub_and_fetch(ptr, val);
}

static __inline int32_t nlib_atomic_and_fetch32(int32_t* ptr, int32_t val,
                                                int memorder) {
    (void)memorder;
    return __sync_and_and_fetch(ptr, val);
}

static __inline int32_t nlib_atomic_xor_fetch32(int32_t* ptr, int32_t val,
                                                int memorder) {
    (void)memorder;
    return __sync_xor_and_fetch(ptr, val);
}

static __inline int32_t nlib_atomic_or_fetch32(int32_t* ptr, int32_t val,
                                               int memorder) {
    (void)memorder;
    return __sync_or_and_fetch(ptr, val);
}

static __inline int32_t nlib_atomic_fetch_add32(int32_t* ptr, int32_t val,
                                                int memorder) {
    (void)memorder;
    return __sync_fetch_and_add(ptr, val);
}

static __inline int32_t nlib_atomic_fetch_sub32(int32_t* ptr, int32_t val,
                                                int memorder) {
    (void)memorder;
    return __sync_fetch_and_sub(ptr, val);
}

static __inline int32_t nlib_atomic_fetch_and32(int32_t* ptr, int32_t val,
                                                int memorder) {
    (void)memorder;
    return __sync_fetch_and_and(ptr, val);
}

static __inline int32_t nlib_atomic_fetch_xor32(int32_t* ptr, int32_t val,
                                                int memorder) {
    (void)memorder;
    return __sync_fetch_and_xor(ptr, val);
}

static __inline int32_t nlib_atomic_fetch_or32(int32_t* ptr, int32_t val,
                                               int memorder) {
    (void)memorder;
    return __sync_fetch_and_or(ptr, val);
}

static __inline int64_t nlib_atomic_load64(const int64_t* ptr, int memorder) {
    int64_t rval = *(volatile int64_t*)ptr;
    (void)memorder;
#if !defined(__i386__) && !defined(__x86_64__)
    if (memorder & NLIB_ATOMIC_ACQUIRE)
        NLIB_MEMORY_ORDER_ACQUIRE;
#endif
    return rval;
}

static NLIB_ALWAYS_INLINE void nlib_atomic_store64(int64_t* ptr, int64_t val,
                                                   int memorder) {
    if (memorder == NLIB_ATOMIC_SEQ_CST)
        __sync_synchronize();
    else if (memorder & NLIB_ATOMIC_RELEASE)
        NLIB_MEMORY_ORDER_RELEASE;
    __sync_lock_test_and_set(ptr, val);
}

static __inline int64_t nlib_atomic_exchange64(int64_t* ptr, int64_t val,
                                               int memorder) {
    if (memorder == NLIB_ATOMIC_SEQ_CST)
        __sync_synchronize();
    else if (memorder & NLIB_ATOMIC_RELEASE)
        NLIB_MEMORY_ORDER_RELEASE;
    return __sync_lock_test_and_set(ptr, val);
}

static __inline void* nlib_atomic_exchangeptr(void** ptr, void* val, int memorder) {
    if (memorder == NLIB_ATOMIC_SEQ_CST)
        __sync_synchronize();
    else if (memorder & NLIB_ATOMIC_RELEASE)
        NLIB_MEMORY_ORDER_RELEASE;
    return __sync_lock_test_and_set(ptr, val);
}

static __inline int nlib_atomic_compare_exchange64(int64_t* ptr, int64_t* expected,
                                                   int64_t desired, int weak,
                                                   int success_memorder, int failure_memorder) {
    int64_t old = __sync_val_compare_and_swap(ptr, *expected, desired);
    if (old == *expected) return 1;
    *expected = old;

    (void)weak;
    (void)success_memorder;
    (void)failure_memorder;
    return 0;
}

static __inline int64_t nlib_atomic_add_fetch64(int64_t* ptr, int64_t val,
                                                int memorder) {
    (void)memorder;
    return __sync_add_and_fetch(ptr, val);
}

static __inline int64_t nlib_atomic_sub_fetch64(int64_t* ptr, int64_t val,
                                                int memorder) {
    (void)memorder;
    return __sync_sub_and_fetch(ptr, val);
}

static __inline int64_t nlib_atomic_and_fetch64(int64_t* ptr, int64_t val,
                                                int memorder) {
    (void)memorder;
    return __sync_and_and_fetch(ptr, val);
}

static __inline int64_t nlib_atomic_xor_fetch64(int64_t* ptr, int64_t val,
                                                int memorder) {
    (void)memorder;
    return __sync_xor_and_fetch(ptr, val);
}

static __inline int64_t nlib_atomic_or_fetch64(int64_t* ptr, int64_t val,
                                               int memorder) {
    (void)memorder;
    return __sync_or_and_fetch(ptr, val);
}

static __inline int64_t nlib_atomic_fetch_add64(int64_t* ptr, int64_t val,
                                                int memorder) {
    (void)memorder;
    return __sync_fetch_and_add(ptr, val);
}

static __inline int64_t nlib_atomic_fetch_sub64(int64_t* ptr, int64_t val,
                                                int memorder) {
    (void)memorder;
    return __sync_fetch_and_sub(ptr, val);
}

static __inline int64_t nlib_atomic_fetch_and64(int64_t* ptr, int64_t val,
                                                int memorder) {
    (void)memorder;
    return __sync_fetch_and_and(ptr, val);
}

static __inline int64_t nlib_atomic_fetch_xor64(int64_t* ptr, int64_t val,
                                                int memorder) {
    (void)memorder;
    return __sync_fetch_and_xor(ptr, val);
}

static __inline int64_t nlib_atomic_fetch_or64(int64_t* ptr, int64_t val,
                                               int memorder) {
    (void)memorder;
    return __sync_fetch_and_or(ptr, val);
}

static __inline void* nlib_atomic_loadptr(void* const* ptr, int memorder) {
    void* rval = *(void* volatile *)ptr;  // NOLINT
    (void)memorder;
#if !defined(__i386__) && !defined(__x86_64__)
    if (memorder & NLIB_ATOMIC_ACQUIRE)
        NLIB_MEMORY_ORDER_ACQUIRE;
#endif
    return rval;
}

static NLIB_ALWAYS_INLINE void nlib_atomic_storeptr(void** ptr, void* val, int memorder) {
    if (memorder == NLIB_ATOMIC_SEQ_CST)
        __sync_synchronize();
    else if (memorder & NLIB_ATOMIC_RELEASE)
        NLIB_MEMORY_ORDER_RELEASE;
    void* tmp = __sync_lock_test_and_set(ptr, val);
    (void)tmp;
}

static __inline int nlib_atomic_compare_exchangeptr(void** ptr, void** expected,
                                                    void* desired, int weak,
                                                    int success_memorder, int failure_memorder) {
    void* old = __sync_val_compare_and_swap(ptr, *expected, desired);
    if (old == *expected) return 1;
    *expected = old;

    (void)weak;
    (void)success_memorder;
    (void)failure_memorder;
    return 0;
}

static __inline void nlib_atomic_thread_fence(int memorder) {
    switch (memorder) {
    case NLIB_ATOMIC_RELAXED:
        break;
    case NLIB_ATOMIC_ACQUIRE:
        NLIB_MEMORY_ORDER_ACQUIRE;
        break;
    case NLIB_ATOMIC_RELEASE:
        NLIB_MEMORY_ORDER_RELEASE;
        break;
    case NLIB_ATOMIC_ACQ_REL:
        NLIB_MEMORY_ORDER_ACQ_REL;
        break;
    default:
        NLIB_MEMORY_ORDER_SEQ_CST;
        break;
    }
}
#endif


#ifdef __cplusplus
}
#endif

#endif
#endif  // INCLUDE_NN_NLIB_PLATFORM_UNIX_H_
