﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_REALLOCOUTPUTSTREAM_H_
#define INCLUDE_NN_NLIB_REALLOCOUTPUTSTREAM_H_

#include <stdlib.h>

#include "nn/nlib/UniquePtr.h"
#include "nn/nlib/OutputStream.h"

NLIB_NAMESPACE_BEGIN

class NLIB_VIS_PUBLIC ReallocOutputStream NLIB_FINAL : public OutputStream {
 public:
    class NLIB_VIS_PUBLIC ReallocDeleter {
     public:
        typedef void* (*ReallocFunc)(void* ptr, size_t n) NLIB_NOEXCEPT_FUNCPTR;
        ReallocDeleter() NLIB_NOEXCEPT : realloc_(::nlib_realloc) {}
        explicit ReallocDeleter(ReallocFunc reallocfunc) NLIB_NOEXCEPT : realloc_(reallocfunc) {}
        void operator()(void* p) const NLIB_NOEXCEPT;
        ReallocFunc GetFunc() const NLIB_NOEXCEPT { return realloc_; }

     private:
        ReallocFunc realloc_;
    };
    NLIB_STATIC_ASSERT(!IsEmpty<ReallocDeleter>::value);
    typedef UniquePtr<uint8_t[], ReallocDeleter> UniquePtrType;
    typedef UniquePtr<char[], ReallocDeleter> CharPtrType;
    typedef void* (*ReallocFunc)(void* ptr, size_t n) NLIB_NOEXCEPT_FUNCPTR;

    ReallocOutputStream() NLIB_NOEXCEPT
        : data_(NULL), realloc_func_(nlib_realloc), capacity_(0) {}
    explicit ReallocOutputStream(ReallocFunc func) NLIB_NOEXCEPT NLIB_NONNULL
        : data_(NULL), realloc_func_(func), capacity_(0) {}
    virtual ~ReallocOutputStream() NLIB_NOEXCEPT NLIB_OVERRIDE;

    uint8_t* Data() NLIB_NOEXCEPT { return data_; }
    const uint8_t* Data() const NLIB_NOEXCEPT { return data_; }

    bool Reserve(size_t n) NLIB_NOEXCEPT { return this->Expand(n) == 0; }
    void ShrinkToFit() NLIB_NOEXCEPT {
        if (this->ExpandOrShrink(this->Pos()) == 0) {
            this->ResetBuffer(data_ + capacity_, 0);
        }
    }

    size_t Release(UniquePtrType* ptr) NLIB_NOEXCEPT;
    // UniquePtrType Reelase(size_t* n);

    // Appends '\0' and, returns the memory after ShrinkToFit().
    bool ReleaseAsCstring(CharPtrType* ptr) NLIB_NOEXCEPT;

 private:
    errno_t Expand(size_t newcap) NLIB_NOEXCEPT {
        return (newcap > capacity_) ? this->ExpandOrShrink(newcap) : 0;
    }
    errno_t ExpandOrShrink(size_t newcap) NLIB_NOEXCEPT;
    virtual bool PushBuffer_(const void* p, size_t nbytes,
                             bool do_flush) NLIB_NOEXCEPT NLIB_OVERRIDE;
    virtual bool Close_() NLIB_NOEXCEPT NLIB_OVERRIDE;
    virtual void* GetWorkBuffer_(size_t* nbytes) NLIB_NOEXCEPT NLIB_OVERRIDE;

 private:
    uint8_t* data_;
    ReallocFunc realloc_func_;
    size_t capacity_;
    NLIB_DISALLOW_COPY_AND_ASSIGN(ReallocOutputStream);
};

NLIB_NAMESPACE_END

#endif  // INCLUDE_NN_NLIB_REALLOCOUTPUTSTREAM_H_
