﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_SWAP_H_
#define INCLUDE_NN_NLIB_SWAP_H_

#include "nn/nlib/Config.h"
#ifdef NLIB_CXX11_SWAPHEADER
# include <utility> // NOLINT
#else
# include <algorithm> // NOLINT
#endif

#if defined(NLIB_CXX11_NOEXCEPT) && defined(NLIB_HAS_NATIVE_TYPETRAITS)
# include <type_traits>  // NOLINT
#define NLIB_DEFINE_STD_SWAP(tp) \
namespace std { template<> inline void swap<tp> (tp& lhs, tp& rhs) noexcept( /* NOLINT */ \
        std::is_nothrow_move_constructible<tp>::value && \
        std::is_nothrow_move_assignable<tp>::value) { lhs.swap(rhs); } }
#else
#define NLIB_DEFINE_STD_SWAP(tp) \
namespace std { template<> inline void swap<tp> \
    (tp& lhs, tp& rhs) { lhs.swap(rhs); } } // NOLINT
#endif

#ifndef NLIB_STD_SWAP_WORKAROUND
# define NLIB_DEFINE_STD_SWAP_T_BEGIN1(ns1) namespace ns1 { // NOLINT
# define NLIB_DEFINE_STD_SWAP_T_BEGIN2(ns1, ns2) namespace ns1 { namespace ns2 { // NOLINT
# define NLIB_DEFINE_STD_SWAP_T_BEGIN3(ns1, ns2, ns3) \
    namespace ns1 { namespace ns2 { namespace ns3 { // NOLINT
# define NLIB_DEFINE_STD_SWAP_T_BEGIN4(ns1, ns2, ns3, ns4) \
    namespace ns1 { namespace ns2 { namespace ns3 { namespace ns4 { // NOLINT
# define NLIB_DEFINE_STD_SWAP_T_END1(ns1) } // NOLINT
# define NLIB_DEFINE_STD_SWAP_T_END2(ns1, ns2) }} // NOLINT
# define NLIB_DEFINE_STD_SWAP_T_END3(ns1, ns2, ns3) }}} // NOLINT
# define NLIB_DEFINE_STD_SWAP_T_END4(ns1, ns2, ns3, ns4) }}}} // NOLINT
# define NLIB_DEFINE_STD_SWAP_T1(targ1, tp) \
    template<class targ1> NLIB_ALWAYS_INLINE void swap \
    (tp<targ1>& lhs, tp<targ1>& rhs) { lhs.swap(rhs); } // NOLINT
# define NLIB_DEFINE_STD_SWAP_T2(targ1, targ2, tp) \
    template<class targ1, class targ2> NLIB_ALWAYS_INLINE void swap \
    (tp<targ1, targ2>& lhs, tp<targ1, targ2>& rhs) { lhs.swap(rhs); } // NOLINT
#else
// This code is for the compilers which std::swap has priority over ADL.
# define NLIB_DEFINE_STD_SWAP_T_BEGIN1(ns1) namespace std { // NOLINT
# define NLIB_DEFINE_STD_SWAP_T_BEGIN2(ns1, ns2) namespace std { // NOLINT
# define NLIB_DEFINE_STD_SWAP_T_BEGIN3(ns1, ns2, ns3) namespace std { // NOLINT
# define NLIB_DEFINE_STD_SWAP_T_BEGIN4(ns1, ns2, ns3, ns4) namespace std { // NOLINT
# define NLIB_DEFINE_STD_SWAP_T_END1(ns1) } // NOLINT
# define NLIB_DEFINE_STD_SWAP_T_END2(ns1, ns2) } // NOLINT
# define NLIB_DEFINE_STD_SWAP_T_END3(ns1, ns2, ns3) } // NOLINT
# define NLIB_DEFINE_STD_SWAP_T_END4(ns1, ns2, ns3, ns4) } // NOLINT
# define NLIB_DEFINE_STD_SWAP_T1(targ1, tp) \
    template<class targ1> NLIB_ALWAYS_INLINE void \
    swap(tp<targ1>& lhs, tp<targ1>& rhs) { lhs.swap(rhs); } // NOLINT
# define NLIB_DEFINE_STD_SWAP_T2(targ1, targ2, tp) \
    template<class targ1, class targ2> NLIB_ALWAYS_INLINE void \
    swap(tp<targ1, targ2>& lhs, tp<targ1, targ2>& rhs) { lhs.swap(rhs); } // NOLINT
#endif

#endif  // INCLUDE_NN_NLIB_SWAP_H_
