﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_EXI_TYPES_H_
#define INCLUDE_NN_NLIB_EXI_TYPES_H_

#include "nn/nlib/Config.h"
#include "nn/nlib/Cstring.h"
#include "nn/nlib/unicode/Convert.h"

#if defined(_MSC_VER) || defined(CAFE) || (defined(__GNUC__) && !defined(__clang__))
#define NLIB_EXI_WCHAR_INTERFACE
#endif

#if defined(_MSC_VER) && defined(nx_exi_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINEXPORT
#endif

NLIB_NAMESPACE_BEGIN
namespace exi {

#ifdef NLIB_EXI_WCHAR_INTERFACE
typedef wchar_t ExiChar;
#define NLIB_EXI_ESTR L""
#define NLIB_EXI_STR(x) L##x
#else
typedef nlib_utf8_t ExiChar;
#define NLIB_EXI_ESTR ""
#define NLIB_EXI_STR(x) x
#endif

enum Alignment { ALIGNMENT_BIT_PACKED = 0, ALIGNMENT_BYTE_ALIGNMENT = 1 };

struct NLIB_VIS_PUBLIC Preserve {
    bool comments;
    bool pis;
    bool prefixes;
    char _;

 public:
    NLIB_CEXPR Preserve() NLIB_NOEXCEPT : comments(false), pis(false), prefixes(false), _(0) {}
    NLIB_CEXPR Preserve(bool comments_, bool pis_, bool prefixes_) NLIB_NOEXCEPT
        : comments(comments_),
          pis(pis_),
          prefixes(prefixes_),
          _(0) {}
};

enum XmlProcessor { XML_PROCESSOR_EXI = 0, XML_PROCESSOR_TEXT };

namespace core {

enum EventType {
    NEXI_START_DOCUMENT = 0,
    NEXI_ATTRIBUTE,
    NEXI_ATTRIBUTE_UNDECLARED,
    NEXI_START_ELEMENT,
    NEXI_START_ELEMENT_UNDECLARED,
    NEXI_START_ELEMENT_UNDECLARED_DOCCONTENT,
    NEXI_CHARACTERS,
    NEXI_CHARACTERS_UNDECLARED,
    NEXI_END_ELEMENT,
    NEXI_END_ELEMENT_UNDECLARED,
    NEXI_COMMENT,
    NEXI_PROCESSING_INSTRUCTION,
    NEXI_NAMESPACE,
    NEXI_END_DOCUMENT,
    NEXI_INVALID_EVENT
};

struct NBitUint {
    int nb;     // # of bits needed for encoding
    int value;  // the value encoded
};

struct EventCode {
    NBitUint ec1;
    NBitUint ec2;
    NBitUint ec3;
};
}  // namespace core

#ifdef _MSC_VER

#ifdef NLIB_EXI_WCHAR_INTERFACE
#define NLIB_EXI_LITERAL(x) NLIB_EXI_STR(x)
#define NLIB_EXI_UTF8(x) ::nlib_ns::unicode::ToUtf8Obj(x).c_str()
#define NLIB_EXI_STRING(x) ::nlib_ns::unicode::ToWideObj(x).c_str()
#else
#define NLIB_EXI_LITERAL(x) ::nlib_ns::unicode::ToUtf8Obj(L##x).c_str()
#define NLIB_EXI_UTF8(x) x
#define NLIB_EXI_STRING(x) ::nlib_ns::unicode::ToUtf8Obj(x).c_str()
#endif

#else

#define NLIB_EXI_LITERAL(x) NLIB_EXI_STR(x)
#ifdef NLIB_EXI_WCHAR_INTERFACE
#define NLIB_EXI_UTF8(x) ::nlib_ns::unicode::ToUtf8Obj(x).c_str()
#define NLIB_EXI_STRING(x) ::nlib_ns::unicode::ToWideObj(x).c_str()
#else
#define NLIB_EXI_UTF8(x) x
#define NLIB_EXI_STRING(x) ::nlib_ns::unicode::ToUtf8Obj(x).c_str()
#endif

#endif

}  // namespace exi
NLIB_NAMESPACE_END

#if defined(_MSC_VER) && defined(nx_exi_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINIMPORT
#endif

#endif  // INCLUDE_NN_NLIB_EXI_TYPES_H_
