﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_MSGPACK_MPWALKER_H_
#define INCLUDE_NN_NLIB_MSGPACK_MPWALKER_H_

#include "nn/nlib/Config.h"
#include "nn/nlib/Swap.h"

#if defined(_MSC_VER) && defined(nx_msgpack_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINEXPORT
#endif

NLIB_NAMESPACE_BEGIN
namespace msgpack {

class MpObject;
class NLIB_VIS_PUBLIC MpWalker NLIB_FINAL {
 public:
    MpWalker() NLIB_NOEXCEPT : cur_(NULL), size_(0) {}
    ~MpWalker() NLIB_NOEXCEPT {}
    MpWalker(const void* p, size_t n) NLIB_NOEXCEPT;
    bool Init(const void* p, size_t n) NLIB_NOEXCEPT NLIB_NONNULL {
        if (n > RSIZE_MAX) return false;
        cur_ = reinterpret_cast<const uint8_t*>(p);
        size_ = n;
        return true;
    }

    MpWalker operator[](size_t array_idx) const NLIB_NOEXCEPT;
    MpWalker operator[](const nlib_utf8_t* key) const NLIB_NOEXCEPT NLIB_NONNULL;
    MpWalker operator[](int array_idx) const NLIB_NOEXCEPT {
        if (array_idx < 0)
            return MpWalker();
        else
            return operator[](static_cast<size_t>(array_idx));
    }

    const void* GetPtr() const NLIB_NOEXCEPT { return cur_; }
    size_t GetSize() const NLIB_NOEXCEPT { return size_; }

    MpWalker At(size_t idx) const NLIB_NOEXCEPT;
    MpWalker At(size_t idx, const nlib_utf8_t** key, size_t* n) const NLIB_NOEXCEPT NLIB_NONNULL;
    MpWalker Find(const nlib_utf8_t* key) const NLIB_NOEXCEPT NLIB_NONNULL;
    MpWalker Skip() const NLIB_NOEXCEPT;
    MpWalker GetString(const nlib_utf8_t** str, uint32_t* n) const NLIB_NOEXCEPT NLIB_NONNULL;
    MpWalker GetBinary(const void** bin, uint32_t* n) const NLIB_NOEXCEPT NLIB_NONNULL;
    MpWalker GetExt(int8_t* tp, const void** bin, uint32_t* n) const NLIB_NOEXCEPT NLIB_NONNULL;
    MpWalker GetMapCount(uint32_t* n) const NLIB_NOEXCEPT NLIB_NONNULL;
    MpWalker GetArrayCount(uint32_t* n) const NLIB_NOEXCEPT NLIB_NONNULL;
    MpWalker GetNil() const NLIB_NOEXCEPT;
    MpWalker GetBoolean(bool* val) const NLIB_NOEXCEPT NLIB_NONNULL;
    MpWalker GetInt(int32_t* val) const NLIB_NOEXCEPT NLIB_NONNULL;
    MpWalker GetInt(int64_t* val) const NLIB_NOEXCEPT NLIB_NONNULL;
    MpWalker GetUint(uint32_t* val) const NLIB_NOEXCEPT NLIB_NONNULL;
    MpWalker GetUint(uint64_t* val) const NLIB_NOEXCEPT NLIB_NONNULL;
    MpWalker GetFloat(float* val) const NLIB_NOEXCEPT NLIB_NONNULL;
    MpWalker GetDouble(double* val) const NLIB_NOEXCEPT NLIB_NONNULL;
    NLIB_SAFE_BOOL(MpWalker, IsOk())

 private:
    bool IsOk() const NLIB_NOEXCEPT { return cur_ != NULL; }

    const uint8_t* cur_;
    size_t size_;
};

}  // namespace msgpack
NLIB_NAMESPACE_END

#if defined(_MSC_VER) && defined(nx_msgpack_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINIMPORT
#endif

#endif  // INCLUDE_NN_NLIB_MSGPACK_MPWALKER_H_
