﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_MSGPACK_JSONRPC_JSONRPCRESPONSE_H_
#define INCLUDE_NN_NLIB_MSGPACK_JSONRPC_JSONRPCRESPONSE_H_

#include "nn/nlib/Config.h"
#include "nn/nlib/UniquePtr.h"
#include "nn/nlib/Nlist.h"
#include "nn/nlib/Swap.h"
#include "nn/nlib/ReallocOutputStream.h"
#include "nn/nlib/msgpack/MpObject.h"

#if defined(_MSC_VER) && defined(nx_msgpack_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINEXPORT
#endif

NLIB_NAMESPACE_BEGIN
namespace msgpack {
namespace jsonrpc {
typedef uint32_t reqid_t;

class NLIB_VIS_PUBLIC JsonRpcResponse NLIB_FINAL {
 public:
    enum ErrorCode {
        kOk = 0,
        kParseError = -32700,
        kInvalidRequest = -32600,
        kMethodNotFound = -32601,
        kInvalidParams = -32602,
        kInternalError = -32603,
        kClientAbort = -31000,
        OK = kOk,
        PARSE_ERROR = kParseError,
        INVALID_REQUEST = kInvalidRequest,
        METHOD_NOT_FOUND = kMethodNotFound,
        INVALID_PARAMS = kInvalidParams,
        INTERNAL_ERROR = kInternalError,
        CLIENT_ABORT = kClientAbort
    };
    JsonRpcResponse() NLIB_NOEXCEPT
        : id_(0), is_error_(false), error_code_(0), error_msg_(NULL) {}
    ~JsonRpcResponse() NLIB_NOEXCEPT;

    void MoveResultFrom(MpObject& obj) NLIB_NOEXCEPT;  // NOLINT
    void SetError(int errcode, const char* msg, MpObject& data) NLIB_NOEXCEPT;  // NOLINT
    void SetError(int errcode, const char* msg) NLIB_NOEXCEPT {
        MpObject data;
        this->SetError(errcode, msg, data);
    }

    ErrorCode GetError() const NLIB_NOEXCEPT {
        return static_cast<ErrorCode>(error_code_);
    }
    reqid_t GetId() const NLIB_NOEXCEPT { return id_; }
    const char* GetErrorMessage() const NLIB_NOEXCEPT {
        return error_msg_ ? error_msg_ : "";
    }
    MpObject& GetMpObject() NLIB_NOEXCEPT { return obj_; }
    const MpObject& GetMpObject() const NLIB_NOEXCEPT { return obj_; }
    NLIB_SAFE_BOOL(JsonRpcResponse, !is_error_);

 private:
    void SetId(reqid_t id) NLIB_NOEXCEPT { id_ = id; }  // from JsonRpcRequest::Init()
    NLIB_VIS_HIDDEN errno_t Init(MpObject* rhs) NLIB_NOEXCEPT;  // from JsonRpcResponseReader

 private:
    reqid_t id_;
    bool is_error_;
    int error_code_;
    char* error_msg_;
    MpObject obj_;

    friend class JsonRpcResponseReader;
    friend class JsonRpcRequest;
    NLIB_DISALLOW_COPY_AND_ASSIGN(JsonRpcResponse);
};

class NLIB_VIS_PUBLIC JsonRpcResponseReader {
 public:
    typedef UniquePtr<JsonRpcResponse> ValueType;
    typedef Nlist<ValueType> ListType;
    static errno_t ReadResponse(const void* p, size_t n, ListType* result_list) NLIB_NOEXCEPT;
};

class NLIB_VIS_PUBLIC JsonRpcResponseWriter NLIB_FINAL {
 public:
    JsonRpcResponseWriter() NLIB_NOEXCEPT : prv_(NULL), is_msgpack_(false) {}
    ~JsonRpcResponseWriter() NLIB_NOEXCEPT;
    NLIB_MOVE_MEMBER_HELPER_1(JsonRpcResponseWriter, prv_);
    explicit JsonRpcResponseWriter(bool msgpack) NLIB_NOEXCEPT
        : prv_(NULL), is_msgpack_(msgpack) {}
    errno_t BeginWriteResponse(uint32_t n) NLIB_NOEXCEPT;
    errno_t WriteResponse(JsonRpcResponse* result) NLIB_NOEXCEPT;
    errno_t EndWriteResponse(ReallocOutputStream::UniquePtrType* ptr, size_t* n) NLIB_NOEXCEPT;
    void swap(JsonRpcResponseWriter& rhs) NLIB_NOEXCEPT {
        using std::swap;
        swap(prv_, rhs.prv_);
        swap(is_msgpack_, rhs.is_msgpack_);
    }

 private:
    struct JsonRpcResponseWriterPrivate;
    JsonRpcResponseWriterPrivate* prv_;
    bool is_msgpack_;
    NLIB_DISALLOW_COPY_AND_ASSIGN(JsonRpcResponseWriter);
};

}  // namespace jsonrpc
}  // namespace msgpack
NLIB_NAMESPACE_END
NLIB_DEFINE_STD_SWAP(::nlib_ns::msgpack::jsonrpc::JsonRpcResponseWriter)
#if defined(_MSC_VER) && defined(nx_msgpack_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINIMPORT
#endif

#endif  // INCLUDE_NN_NLIB_MSGPACK_JSONRPC_JSONRPCRESPONSE_H_
