﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_SUCCINCT_AHOCORASICKBUILDER_H_
#define INCLUDE_NN_NLIB_SUCCINCT_AHOCORASICKBUILDER_H_

#include <string.h>
#include <utility>

#include "nn/nlib/Swap.h"
#include "nn/nlib/succinct/AhoCorasick.h"

#if defined(_MSC_VER) && defined(nx_succinct_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINEXPORT
#endif

NLIB_NAMESPACE_BEGIN
namespace succinct {

class NLIB_VIS_PUBLIC AhoCorasickBuilder NLIB_FINAL {
 public:
    AhoCorasickBuilder() NLIB_NOEXCEPT : prv_(NULL) {}
    ~AhoCorasickBuilder() NLIB_NOEXCEPT;
    NLIB_MOVE_MEMBER_HELPER_1(AhoCorasickBuilder, prv_);
    bool Init() NLIB_NOEXCEPT;
    bool AddWord(const char* str) NLIB_NOEXCEPT;
    bool AddPattern(const void* p, size_t n) NLIB_NOEXCEPT;
    bool AddWords(const char* str, size_t len) NLIB_NOEXCEPT;
    bool AddWords(const char* str) NLIB_NOEXCEPT {
        return AddWords(str, nlib_strlen(str));
    }
    AhoCorasick* Build() NLIB_NOEXCEPT;
    typedef bool (*MatchCallback)(const char* first, const char* last, uint32_t nodeid,
                                  void* user_obj);
    void MatchByBuilder(const char* doc, MatchCallback callback, void* user_obj) NLIB_NOEXCEPT;
    void MatchByBuilder(const char* doc, MatchCallback callback) NLIB_NOEXCEPT {
        MatchByBuilder(doc, callback, NULL);
    }
    void print() NLIB_NOEXCEPT;
    size_t GetNumWords() const NLIB_NOEXCEPT;
    size_t GetNumBytes() const NLIB_NOEXCEPT;
    size_t GetNumNodes() const NLIB_NOEXCEPT;
    void swap(AhoCorasickBuilder& rhs) NLIB_NOEXCEPT {
        using std::swap;
        swap(prv_, rhs.prv_);
    }

 private:
    NLIB_VIS_HIDDEN bool SortNodes() NLIB_NOEXCEPT;
    struct BuildFailureArcTh;
    struct BuildReportTreeHolderTh;
    struct BuildFailureTreeHolderTh;
    struct BuildGotoArcHolderTh;
    struct BuildLenHolderTh;
    struct SortNodesTh;

 private:
    struct AhoCorasickBuilderPrivate;
    AhoCorasickBuilderPrivate* prv_;
    NLIB_DISALLOW_COPY_AND_ASSIGN(AhoCorasickBuilder);
};

}  // namespace succinct
NLIB_NAMESPACE_END
NLIB_DEFINE_STD_SWAP(::nlib_ns::succinct::AhoCorasickBuilder)
#if defined(_MSC_VER) && defined(nx_succinct_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINIMPORT
#endif

#endif  // INCLUDE_NN_NLIB_SUCCINCT_AHOCORASICKBUILDER_H_
