﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_THREADING_TLS_H_
#define INCLUDE_NN_NLIB_THREADING_TLS_H_

#include "nn/nlib/Config.h"

NLIB_NAMESPACE_BEGIN
namespace threading {

template <class T>
class Tls NLIB_FINAL {
 public:
    NLIB_CEXPR Tls() NLIB_NOEXCEPT : idx_(), is_idx_valid_(false) {}
    errno_t Init() NLIB_NOEXCEPT {
        if (is_idx_valid_) return EALREADY;
        errno_t e = nlib_tls_alloc(&idx_, destr_func);
        if (e == 0) is_idx_valid_ = true;
        return e;
    }
    ~Tls() NLIB_NOEXCEPT {
        if (is_idx_valid_) {
            nlib_tls_free(idx_);
            is_idx_valid_ = false;
        }
    }
    T* Get() const NLIB_NOEXCEPT {
        void* p;
        errno_t e = nlib_tls_getvalue(idx_, &p);
        if (e != 0) return NULL;
        return reinterpret_cast<T*>(p);
    }
    T* operator->() const NLIB_NOEXCEPT { return this->Get(); }
    T& operator*() const NLIB_NOEXCEPT { return *this->Get(); }
    T* Release() NLIB_NOEXCEPT {
        T* obj = this->Get();
        nlib_tls_setvalue(idx_, NULL);
        return obj;
    }
    errno_t Reset(T* p = 0) NLIB_NOEXCEPT {
        T* obj = this->Get();
        if (obj) {
            delete obj;
        }
        return nlib_tls_setvalue(idx_, p);
    }

 private:
    static void destr_func(void* p) NLIB_NOEXCEPT {
        delete reinterpret_cast<T*>(p);
    }
    nlib_tls idx_;
    bool is_idx_valid_;
};

}  // namespace threading
NLIB_NAMESPACE_END

#endif  // INCLUDE_NN_NLIB_THREADING_TLS_H_
