Param([string]$target,
      [string]$CMake = "cmake.exe")
Push-Location .
$scriptDir = Split-Path -Parent $MyInvocation.MyCommand.Path
Set-Location $scriptDir

Set-Item Env:Path ((($Env:Path -split ';') -match '^(?!.*cygwin)') -join ';')

$dll = ""
switch($target) {
    "vc120" {
        $gen = "Visual Studio 12 2013"
    }
    "vc120amd64" {
        $gen = "Visual Studio 12 2013 Win64"
    }
    "vc120dllamd64" {
        $gen = "Visual Studio 12 2013 Win64"
        $dll = "-D NLIB_DLL=1"
    }
    "vc140" {
        $gen = "Visual Studio 14 2015"
    }
    "vc140amd64" {
        $gen = "Visual Studio 14 2015 Win64"
    }
    "vc140dllamd64" {
        $gen = "Visual Studio 14 2015 Win64"
        $dll = "-D NLIB_DLL=1"
    }
    "vc150amd64" {
        $gen = "Visual Studio 15 2017 Win64"
    }
    "vc150dllamd64" {
        $gen = "Visual Studio 15 2017 Win64"
        $dll = "-D NLIB_DLL=1"
    }
    default {
        if (Test-Path "HKCU:\SOFTWARE\Microsoft\VisualStudio\15.0") {
            $target = "vc150amd64"
            $gen = "Visual Studio 15 2017 Win64"
        } elseif (Test-Path "HKCU:\SOFTWARE\Microsoft\VisualStudio\14.0") {
            $target = "vc140amd64"
            $gen = "Visual Studio 14 2015 Win64"
        } elseif (Test-Path "HKCU:\SOFTWARE\Microsoft\VisualStudio\12.0") {
            $target = "vc120amd64"
            $gen = "Visual Studio 12 2013 Win64"
        } else {
            Write-Error "No Visual Studio found."
            exit 1
        }
    }
}

Push-Location .
try {
    New-Item $target\Release -ItemType Directory
    Set-Location $target\Release
    & $CMake -G $gen $dll -D CMAKE_BUILD_TYPE=Release ..\..\
    if ($LastExitCode -ne 0) { throw }
} catch {
    Set-Location ..\..\
    Remove-Item $target\Release -Recurse -Force
    exit 1
} finally {
    Pop-Location
}

Push-Location .
try {
    New-Item $target\Debug -ItemType Directory
    Set-Location $target\Debug
    & $CMake -G $gen $dll -D CMAKE_BUILD_TYPE=Debug ..\..\
    if ($LastExitCode -ne 0) { throw }
} catch {
    Set-Location ..\..\
    Remove-Item $target\Debug -Recurse -Force
    exit 1
} finally {
    Pop-Location
}

Pop-Location
